#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import os
from http import HTTPStatus
from AiDrawing import AiDrawing
from PyQt5.QtCore import QSize, pyqtSignal, Qt, QThread, QTimer, QEventLoop
from Config import Config
from XfAiDrawing import XfAiDrawing

current_user = os.getlogin()

class AiDrawingTask(QThread):
    complete_signal = pyqtSignal(str)
    prompt = ""

    def __init__(self):
        super(AiDrawingTask, self).__init__()
        self.prompt = ""
        self.task_result = None
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.timeout_handle)
        path = '/home/'+current_user+'/图片/'
        if not os.path.exists(path):
            print("目录：" + path + " 不存在")
            os.mkdir(path)

    def start_timer(self, time):
        self.timer.start(time)

    def timeout_handle(self):
        print("timeout_handle")
        self.timer.stop()
        if self.task_result:
            status = AiDrawing.fetch_task_status(self.task_result)
            if status == "PENDING":
                AiDrawing.cancel_task(self.task_result)
            elif status == "RUNNING":
                print("Task can't Cancel.")
            # self.task_info = None

    def set_prompt(self, prompt):
        self.prompt = prompt

    def run(self):
        model = Config.get_drawing_model_name()
        if model == "XINGHUO":
            code, content = XfAiDrawing.create_sync_task(self.prompt)
            if code == 0:
                self.complete_signal.emit(content)
            else:
                self.complete_signal.emit("fail: " + content)
        elif model == "TONGYI":
            self.task_result = AiDrawing.create_async_task(self.prompt)
            if self.task_result:
                if self.task_result.status_code == HTTPStatus.OK:
                    AiDrawing.fetch_task_status(self.task_result)
                    file_path = AiDrawing.wait_task(self.task_result)
                    if file_path:
                        self.complete_signal.emit(file_path)
                else:
                    print('Failed, status_code: %s, code: %s, message: %s' %
                          (self.task_result.status_code, self.task_result.code, self.task_result.message))
                    self.complete_signal.emit("fail: " + self.task_result.message)

        else:
            print("model undefine")

