#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import sys
from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QVBoxLayout, QPushButton
from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtCore import QTimer, QSize

class AudioInputWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.isPlaying = False  # 控制动画播放状态
        self.initUI()

    def initUI(self):
        # self.setWindowTitle('图片序列播放器')
        self.setGeometry(100, 100, 300, 300)

        # self.label = QLabel(self)
        self.playButton = QPushButton(self)
        self.playButton.setFixedSize(64,64)
        # self.layout = QVBoxLayout(self)
        # self.layout.addWidget(self.label)
        # self.layout.addWidget(self.playButton)

        self.current_image_index = 0
        self.total_images = 74  # 图片总数

        self.timer = QTimer(self)  # 定义定时器
        self.timer.timeout.connect(self.updateImage)  # 定时器信号连接到updateImage方法
        self.timer.start(100)
        # self.playButton.clicked.connect(self.toggleAnimation)
        # self.playButton.setMinimumHeight(64)

        self.show()

    def updateImage(self):
        print('updateImage')
        if self.current_image_index > self.total_images:
            self.current_image_index = 0  # 重置索引

        image_path = f"./data/光环/光环_{self.current_image_index:05d}.png"
        pixmap = QPixmap(image_path)

        if not pixmap.isNull():
            # self.label.setPixmap(pixmap)
            self.playButton.setIcon(QIcon(image_path))
            self.playButton.setIconSize(QSize(64, 64)); 

            self.current_image_index += 1
        else:
            print(f"图片加载失败: {image_path}")
            self.current_image_index += 1

    def toggleAnimation(self):
        if self.isPlaying:
            self.timer.stop()
            self.playButton.setText('播放动画')
        else:
            self.timer.start(50)  # 100ms更新一次图像
            self.playButton.setText('停止动画')
        self.isPlaying = not self.isPlaying

if __name__ == '__main__':
    app = QApplication(sys.argv)
    ex = AudioInputWidget()
    ex.show()
    sys.exit(app.exec_())
