#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import json

from PyQt5.QtWidgets import (
    QWidget, QLabel, QVBoxLayout, QHBoxLayout, QLineEdit, QPushButton, QApplication, QDesktopWidget, QComboBox, QTabWidget, QProgressBar,QProgressDialog
)
from PyQt5.QtCore import Qt, QSize, pyqtSignal
from PyQt5.QtGui import QIcon, QPainter, QColor, QLinearGradient, QPen
from Config import Config
from PluginsInstall import PluginsInstallTask
from GetInstallStatus import GetInstallStatus

#isClear用于判断点击确认后，是否要清空输入框
isClear = False
class ApiKeyConfigPage(QWidget):
    # 定义打开信号
    # showSettingsPage = pyqtSignal()
    def __init__(self, parent=None):
        super(ApiKeyConfigPage, self).__init__(parent)
        layout = QVBoxLayout(self)  # 包含顶栏和主要内容
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog | Qt.WindowStaysOnTopHint)   # Qt.Dialog主要用于在任务栏隐藏图标
        self.resize(408, 379)
        
        # 背景透明
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        
        # 顶部栏
        top_bar = self.create_top_bar()
        layout.addWidget(top_bar)
        
        # 主要内容
        self.setting_main_window = ApiKeyConfigMainWindow()
        self.setting_main_window.confirm_button.clicked.connect(self.openSettingPage)
        layout.addWidget(self.setting_main_window)

        layout.setContentsMargins(0, 0, 0, 0)
        
           
    def create_top_bar(self):
        setting_page_title = BoxTitle(self)

        #创建顶部栏
        top_bar_layout = QHBoxLayout()

        title_label = QLabel("云端模型")
        title_label.setStyleSheet("font-size: 16px; color: black;")

        close_btn = QPushButton()
        close_btn.setFixedSize(24, 24)
        close_btn.setStyleSheet(
            '''
            QPushButton{background-color:transparent;border:none;color:white;}
            '''
        )

        close_btn.setIcon(QIcon('./data/close.png'))
        close_btn.setIconSize(QSize(24, 24))
        close_btn.clicked.connect(self.close)
        close_btn.setFixedSize(24, 24)

        top_bar_layout.addWidget(title_label)
        top_bar_layout.addStretch(1)
        top_bar_layout.addWidget(close_btn)

        setting_page_title.setLayout(top_bar_layout) 
        
        return setting_page_title
    
    
    def move_to_center(self):
        #移动到窗口中心位置
        # 获取屏幕总数和屏幕几何数据
        desktop = QDesktopWidget()
        screen_count = desktop.screenCount()
        if screen_count > 1:
            # 多屏情况下，用第一个屏幕
            target_screen_index = 0
            # 获取目标屏幕的几何信息
            target_screen_geometry = desktop.screenGeometry(target_screen_index)
        else:
            # 单屏情况下使用默认屏幕
            target_screen_geometry = desktop.screenGeometry()

        # 计算窗口移动位置到屏幕右下角
        right = target_screen_geometry.right()
        bottom = target_screen_geometry.bottom()
        self.move(int(right/2 - self.width()/2), int(bottom/2 - self.height()/2))
        
    #鼠标移动，拖拽窗口实现
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.moving = True
            self.offset = event.pos()

    def mouseMoveEvent(self, event):
        if(self.moving):
            self.move(event.globalPos() - self.offset)

    def mouseReleaseEvent(self, event):
        if(event.button() == Qt.LeftButton):
            self.moving = False

    def paintEvent(self, event):
        #画圆角
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)  # 设置抗锯齿，让圆角更加平滑
        self.setAttribute(Qt.WA_TranslucentBackground, False)
        painter.setPen(QColor(0, 0, 0, 10))  # 黑色，透明度设置为10
        painter.setBrush(self.palette().window().color())#设置画刷为窗口背景颜色
        radius = 15 #设置圆角半径

        bounds = self.rect()  # 获取窗口的边界
        # 绘制阴影
        shadow = QLinearGradient(bounds.topLeft(), bounds.bottomLeft())
        shadow.setColorAt(0, QColor(100, 100, 100, 50))

        # 设置阴影的模糊半径
        painter.setPen(QPen(shadow, 1))
        painter.drawRoundedRect(self.rect(), radius, radius)

    def openSettingPage(self):
        # self.showSettingsPage.emit()
        # print("信号已经发送")
        self.close()
        self.parent().setting_main_window.update_status()
        self.parent().show()


class BoxTitle(QWidget):
    def __init__(self, *args, **kwargs):
        super(BoxTitle, self).__init__()
        self.setAttribute(Qt.WA_StyledBackground)
        self.setObjectName('box_title')
        self.setStyleSheet(
                '''
                #box_title{
                    border-left: 2px solid transparent;
                    border-top-left-radius: 15px;
                    border-top-right-radius: 15px;
                    background-color: white;
                    background-clip: padding-box; 
                }
                '''
            )
        self.installEventFilter(self)
        self.setFixedHeight(36)


class ApiKeyConfigMainWindow(QWidget):
    def __init__(self, parent=None):
        super(ApiKeyConfigMainWindow, self).__init__(parent)
        self.resize(408, 379)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog | Qt.WindowStaysOnTopHint) #Qt.Dialog主要用于在任务栏隐藏图标

        # 背景透明
        self.setAttribute(Qt.WA_TranslucentBackground, True)

        # 实现圆角
        self.setObjectName('mainwindow')
        self.radius = 15

        # 内容框
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(24, 30, 24, 0)

        self.comboBox = QComboBox(self)
        self.comboBox.setFixedSize(360, 42)

        self.comboBox.setStyleSheet("""
                   QComboBox {
                       border: 1px solid #cccccc;
                       border-radius: 6px;  /* 设置圆角半径 */
                       padding: 1px 18px 1px 3px;  /* 设置内边距以留出下拉箭头空间 */
                       font: 14px;
                       width: 42px;
                   }

                   QComboBox::drop-down {
                       subcontrol-origin: padding;
                       subcontrol-position: top right;
                       width: 42px;
                       border-left-width: 0px;
                       border-left-color: #cccccc;
                       border-left-style: solid;
                       border-radius: 6px;  /* 设置圆角半径 */
                   }

                   QComboBox::down-arrow {
                       image: url("./data/icon_dropdown.png");  /* 可选：使用自定义下拉箭头图标 */
                   }

                   QComboBox QAbstractItemView {
                        border: 1px solid #cccccc;
                        background-color: white;
                        selection-background-color: lightgray;
                        border-radius: 6px;
                   }

                   /* 可选：为下拉列表中的项设置样式 */
                    QComboBox QAbstractItemView::item {
                        border-radius: 6px;  /* 设置圆角半径 */
                        padding: 5px;
                    }

                    /* 可选：为下拉列表中的选中项设置样式 */
                    QComboBox QAbstractItemView::item:selected {
                        background-color: lightgray;
                    }
               """)
        mode_list = Config.get_model_list()
        mode = Config.get_model_name()
        print(f"mode list: {mode_list}")
        self.data_list = json.loads(mode_list)

        main_layout.addWidget(self.comboBox)

        self.tabWdg = QTabWidget()
        self.tabWdg.setMinimumHeight(220)
        main_layout.addWidget(self.tabWdg)

        tab_bar = self.tabWdg.tabBar()
        tab_bar.setHidden(True)
        # 背景透明
        self.tabWdg.setStyleSheet("QTabWidget::pane { border: 0px;}")


        # 定义服务配置项的字典,存储placeholder，后续点击清空的时候需要用到
        self.tongyi_service_keys = {
            "api-key": "API Key",
            "app-key": "App Key",
            "access key id": "Access Key ID",
            "access key secret": "Access Key Secret"
        }

        self.xf_service_keys = {
            "xf-app-id": "App ID",
            "xf-api-key": "Api Key",
            "xf-api-secret": "Api Secret",
            "xf-api-password": "Api Password"
        }

        self.ds_service_keys = {
            "ds-api-key": "Api Key",
        }

        self.tongyi_service_box, self.tongyi_inputs = self.create_service_box(
            "通义大模型服务", [("api-key", "API Key"), ("app-key", "App Key"),
                                  ("access key id", "Access Key ID"), ("access key secret", "Access Key Secret")]
        )
        self.tabWdg.addTab(self.tongyi_service_box, 'TONGYI')

        # self.tongyi_service_box, self.tongyi_inputs = self.create_service_box(
        #     "通义听悟·会议纪要服务",
        #     [("app-key", "App Key"), ("access key id", "Access Key ID"),
        #      ("access key secret", "Access Key Secret")])
        #
        # #加入主布局中
        # main_layout.addWidget(self.tongyi_service_box)

        self.xf_service_box, self.xf_inputs = self.create_service_box(
            "星火大模型服务",
            [("xf-app-id", "App ID"), ("xf-api-key", "Api Key"),
             ("xf-api-secret", "Api Secret"),  ("xf-api-password", "Api Password")])
        self.tabWdg.addTab(self.xf_service_box, 'XINGHUO')

        self.ds_service_box, self.ds_inputs = self.create_service_box(
            "DeepSeek大模型服务",
            [("ds-api-key", "Api Key")])
        self.tabWdg.addTab(self.ds_service_box, 'DEEPSEEK')

        for item in self.data_list:
            for key, value in item.items():
                print(f"{key}: {value}")
                if mode == key:
                    self.comboBox.insertItem(0, value)
                else:
                    self.comboBox.addItem(value)

        self.comboBox.currentIndexChanged.connect(self.onComboBoxChanged)

        # 确定和清空框
        button_layout = self.create_buttons()
        main_layout.addLayout(button_layout)

    def onComboBoxChanged(self, index):
        selectedText = self.comboBox.currentText()
        print(f"{selectedText}")
        for item in self.data_list:
            for key, value in item.items():
                if selectedText == value:
                    print(f"{key}")
                    for index in range(self.tabWdg.count()):
                        tab_text = self.tabWdg.tabText(index)
                        if key == tab_text:
                            self.tabWdg.setCurrentIndex(index)


    def create_ai_model_section(self):
        # ai大模型
        ai_model_layout = QVBoxLayout()
        ai_model_layout.setSpacing(15)
        ai_model_layout.setContentsMargins(10, 10, 10, 10)

        ai_model_label = QLabel("AI大模型")
        ai_model_label.setStyleSheet("font-weight: bold;")
        ai_model_layout.addWidget(ai_model_label)

        return ai_model_layout

    def create_service_box(self, service_name, fields):
        # 服务配置项
        service_box = QWidget()
        service_box_layout = QVBoxLayout(service_box)
        service_box_layout.setSpacing(10)

        # 创建一个字典来保存字段名称与对应的 QLineEdit 控件
        field_inputs = {}
        for field_name, placeholder in fields:
            field_label = QLabel(field_name)
            field_input = QLineEdit()

            # 检查配置文件中是否存在该键
            existing_value = None
            if field_name == "api-key":
                existing_value = Config.get_tongyi_api_key()
            elif field_name == "app-key":
                existing_value = Config.get_tongyi_app_key()
            elif field_name == "access key id":
                existing_value = Config.get_tongyi_access_key_id()
            elif field_name == "access key secret":
                existing_value = Config.get_tongyi_access_key_secret()
            elif field_name == "xf-app-id":
                existing_value = Config.get_xf_app_id()
            elif field_name == "xf-api-key":
                existing_value = Config.get_xf_api_key()
            elif field_name == "xf-api-secret":
                existing_value = Config.get_xf_api_secret()
            elif field_name == "xf-api-password":
                existing_value = Config.get_xf_api_password()
            elif field_name == "ds-api-key":
                existing_value = Config.get_ds_api_key()

            # 根据是否存在键来设置 placeholder
            if existing_value:
                field_input.setPlaceholderText(existing_value)
            else:
                field_input.setPlaceholderText(placeholder)

            # 保存输入框的引用
            field_inputs[field_name] = field_input
            service_box_layout.setContentsMargins(2, 10, 2, 2)

            service_box_layout.addWidget(field_input)

       # if (service_name == "通义大模型服务") and (not GetInstallStatus.is_package_installed("curl")):
        if (service_name == "通义大模型服务") and (not GetInstallStatus.is_package_installed("tingwu-meettings-plugin")):
            plugin = QWidget()
            layout = QHBoxLayout(plugin)
            layout.setContentsMargins(2, 0, 2, 2)
            service_label = QLabel("会议记录功能组件需在线安装，")
            service_label.setStyleSheet("font-size:14px;border: 0px solid #cccccc;padding: 0px")
            layout.addWidget(service_label)
            install_btn = QPushButton()
            install_btn.setText("请从源或者软件中心下载安装")
            install_btn.setStyleSheet("color:blue;font-size:14px;border: 0px solid #cccccc;padding: 0px")
            #install_btn.clicked.connect(self.install_plugin)
            layout.addWidget(install_btn)
            service_box_layout.addWidget(plugin)

       # if service_name == "DeepSeek大模型服务":
        service_box_layout.addStretch()

        service_box.setStyleSheet(
            "QWidget { border: none; border-radius: 8px; padding: 10px; }"
        )

        return service_box, field_inputs

    def install_plugin(self):
        print(f"install_plugin")
        install_task = PluginsInstallTask()
        packages = ['wps-office']
        install_task.set_packages(packages)
        install_task.install_pkg()
        # progress_bar = QProgressDialog(self)
        # progress_bar = QProgressBar(self)
        # progress_bar.show()


    def create_buttons(self):
        # 确定和清空按钮
        button_layout = QHBoxLayout()

        clear_button = QPushButton("清空")
        clear_button.setStyleSheet(
            "background-color: #f0f0f0; color: #4d4f53; font-weight: bold; padding: 10px 20px; border-radius: 5px;border:0.5px solid #7d7d7d"
        )
        clear_button.setFixedWidth(114)
        clear_button.clicked.connect(self.clear_inputs)

        self.confirm_button = QPushButton("确认")
        self.confirm_button.setStyleSheet(
            "background-color: #00aaff; color: white; font-weight: bold; padding: 10px 20px; border-radius: 5px;"
        )
        self.confirm_button.setFixedWidth(114)
        self.confirm_button.clicked.connect(self.confirm)

        button_layout.addStretch(1)
        button_layout.addWidget(clear_button)
        button_layout.addWidget(self.confirm_button)
        button_layout.setContentsMargins(0, 48, 0, 24)

        return button_layout

    def confirm(self):
        # 从字段字典中获取 QLineEdit 控件的值
        tongyi_api_key = self.tongyi_inputs['api-key'].text()
        tongyi_app_key = self.tongyi_inputs['app-key'].text()
        tongyi_access_key_id = self.tongyi_inputs['access key id'].text()
        tongyi_access_key_secret = self.tongyi_inputs['access key secret'].text()

        xf_app_id = self.xf_inputs['xf-app-id'].text()
        xf_api_key = self.xf_inputs['xf-api-key'].text()
        xf_api_secret = self.xf_inputs['xf-api-secret'].text()
        xf_api_password = self.xf_inputs['xf-api-password'].text()

        ds_api_key = self.ds_inputs['ds-api-key'].text()

        # 输出结果
        # print(f"百炼 API Key: {tongyi_api_key}")
        # print(f"通义 App Key: {tongyi_app_key}")
        # print(f"通义 Access Key ID: {tongyi_access_key_id}")
        # print(f"通义 Access Key Secret: {tongyi_access_key_secret}")

        global isClear
        # 1、没有点击清空，且输入框为空，则不执行任何操作
        if not isClear and not tongyi_api_key and not tongyi_app_key and not tongyi_access_key_id and not tongyi_access_key_secret \
                and not xf_app_id and not xf_api_key and not xf_api_secret and not xf_api_password and not ds_api_key:
            return

        # 2、点击清空，且输入框为空，则清空配置文件
        if isClear and not tongyi_api_key and not tongyi_app_key and not tongyi_access_key_id and not tongyi_access_key_secret \
                and not xf_app_id and not xf_api_key and not xf_api_secret and not xf_api_password:
            Config.set_tongyi_api_key("")
            Config.set_tongyi_app_key("")
            Config.set_tongyi_access_key_id("")
            Config.set_tongyi_access_key_secret("")
            Config.set_xf_api_key("")
            Config.set_xf_app_id("")
            Config.set_xf_api_secret("")
            Config.set_xf_api_password("")
            Config.set_ds_api_key("")
            return

        # 检查并设置配置，记录成功或失败的状态
        success = True

        # 3、点击确认，输入框有值，则设置配置
        if tongyi_api_key:
            if not Config.set_tongyi_api_key(tongyi_api_key):
                success = False

        if tongyi_app_key:
            if not Config.set_tongyi_app_key(tongyi_app_key):
                success = False

        if tongyi_access_key_id:
            if not Config.set_tongyi_access_key_id(tongyi_access_key_id):
                success = False

        if tongyi_access_key_secret:
            if not Config.set_tongyi_access_key_secret(tongyi_access_key_secret):
                success = False

        if xf_app_id:
            if not Config.set_xf_app_id(xf_app_id):
                success = False

        if xf_api_key:
            if not Config.set_xf_api_key(xf_api_key):
                success = False

        if xf_api_secret:
            if not Config.set_xf_api_secret(xf_api_secret):
                success = False

        if xf_api_password:
            if not Config.set_xf_api_password(xf_api_password):
                success = False
        if ds_api_key:
            if not Config.set_ds_api_key(ds_api_key):
                success = False

    def clear_inputs(self):
        # 清空按钮
        global isClear
        isClear = True
        for widget in self.findChildren(QLineEdit):
            widget.clear()

        # 将 placeholder 文本恢复为原始值
        for field_name in self.tongyi_inputs:
            self.tongyi_inputs[field_name].setPlaceholderText(self.tongyi_service_keys[field_name])

        for field_name in self.xf_inputs:
            self.xf_inputs[field_name].setPlaceholderText(self.xf_service_keys[field_name])

        for field_name in self.ds_inputs:
            self.ds_inputs[field_name].setPlaceholderText(self.ds_service_keys[field_name])


# if __name__ == "__main__":
#     import sys
#     app = QApplication(sys.argv)
#     api_key_config_page = ApiKeyConfigPage()
#     api_key_config_page.show()
#     sys.exit(app.exec_())
