#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import configparser
import json
import dashscope
from PublicTypes import PublicTypes
public_types = PublicTypes()
import os

# 获取家目录路径
home_dir = os.path.expanduser('~')

# 指定配置文件路径
CONFIG_PATH = os.path.join(home_dir, PublicTypes.CONFIG_PATH)

class Config:
    @staticmethod
    def get_text_play_flag():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'text_play')
        # print(api_key)
        return api_key

    @staticmethod
    def set_text_play_flag(model):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('GENERAL', 'text_play', model)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set text play flag: {e}")
            return False

    @staticmethod
    def get_model_name():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'model')
        # print(api_key)
        return api_key

    @staticmethod
    def set_model_name(model):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('GENERAL', 'model', model)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set model: {e}")
            return False

    @staticmethod
    def get_chat_model_name():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'chat_model')
        # print(api_key)
        return api_key

    @staticmethod
    def set_chat_model_name(model):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('GENERAL', 'chat_model', model)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set model: {e}")
            return False

    @staticmethod
    def get_drawing_model_name():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'drawing_model')
        # print(api_key)
        return api_key

    @staticmethod
    def set_drawing_model_name(model):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('GENERAL', 'drawing_model', model)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set model: {e}")
            return False

    @staticmethod
    def get_model_list():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'model_list')
        # print(api_key)
        return api_key

    @staticmethod
    def get_chat_model_list():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'chat_model_list')
        # print(api_key)
        return api_key

    @staticmethod
    def get_drawing_model_list():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('GENERAL', 'drawing_model_list')
        # print(api_key)
        return api_key

    @staticmethod
    def get_app_list():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        data = config.get('GENERAL', 'app_list')
        app_list = json.loads(data)
        return app_list

    @staticmethod
    def get_local_model():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        local_model = config.get('TONGYI', 'local_model')
        # print(local_model)
        return local_model

    @staticmethod
    def get_tongyi_api_key():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('TONGYI', 'api_key')
        return api_key

    @staticmethod
    def set_tongyi_api_key(api_key):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('TONGYI', 'api_key', api_key)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            dashscope.api_key = Config.get_tongyi_api_key()
            return True
        except Exception as e:
            print(f"Failed to set tongyi api key: {e}")
            return False

    @staticmethod
    def get_tongyi_app_key():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        app_key = config.get('TONGYI', 'app_key')
        return app_key

    @staticmethod
    def set_tongyi_app_key(app_key):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('TONGYI', 'app_key', app_key)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set tongyi app key: {e}")
            return False

    @staticmethod
    def get_tongyi_access_key_id():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        access_key_id = config.get('TONGYI', 'access_key_id')
        # print(local_model)
        return access_key_id

    @staticmethod
    def set_tongyi_access_key_id(access_key_id):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('TONGYI', 'access_key_id', access_key_id)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set tongyi access key id: {e}")
            return False

    @staticmethod
    def get_tongyi_access_key_secret():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        access_key_secret = config.get('TONGYI', 'access_key_secret')
        # print(local_model)
        return access_key_secret

    @staticmethod
    def set_tongyi_access_key_secret(access_key_secret):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('TONGYI', 'access_key_secret', access_key_secret)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set tongyi access key secret: {e}")
            return False

    @staticmethod
    def get_tongyi_meeting_plugin_status():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        access_key_secret = config.get('TONGYI', 'meeting_plugin_status')
        # print(local_model)
        return access_key_secret

    @staticmethod
    def set_tongyi_meeting_plugin_status(access_key_secret):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('TONGYI', 'meeting_plugin_status', access_key_secret)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set tongyi meeting plugin status: {e}")
            return False

    @staticmethod
    def get_xf_app_id():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        app_id = config.get('XINGHUO', 'app_id')
        # print(local_model)
        return app_id

    @staticmethod
    def set_xf_app_id(app_id):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('XINGHUO', 'app_id', app_id)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set app id: {e}")
            return False

    @staticmethod
    def get_xf_api_key():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('XINGHUO', 'api_key')
        # print(local_model)
        return api_key

    @staticmethod
    def set_xf_api_key(api_key):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('XINGHUO', 'api_key', api_key)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set api Key: {e}")
            return False

    @staticmethod
    def get_xf_api_secret():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_secret = config.get('XINGHUO', 'api_secret')
        # print(local_model)
        return api_secret

    @staticmethod
    def set_xf_api_secret(api_secret):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('XINGHUO', 'api_secret', api_secret)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set api secret: {e}")
            return False

    @staticmethod
    def get_xf_api_password():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_password = config.get('XINGHUO', 'api_password')
        # print(local_model)
        return api_password

    @staticmethod
    def set_xf_api_password(api_password):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('XINGHUO', 'api_password', api_password)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set api password: {e}")
            return False

    @staticmethod
    def get_ds_api_key():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        api_key = config.get('DEEPSEEK', 'api_key')
        # print(local_model)
        return api_key

    @staticmethod
    def set_ds_api_key(api_key):
        try:
            config = configparser.ConfigParser()
            with open(CONFIG_PATH, "r") as configfile:
                config.read_file(configfile)
            config.set('DEEPSEEK', 'api_key', api_key)
            with open(CONFIG_PATH, "w") as configfile:
                config.write(configfile)
            return True
        except Exception as e:
            print(f"Failed to set api Key: {e}")
            return False

    @staticmethod
    def get_ds_local_model():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        local_model = config.get('DEEPSEEK', 'local_model')
        # print(local_model)
        return local_model

    @staticmethod
    def get_ds_local_model_url():
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        local_model = config.get('DEEPSEEK', 'local_model_url')
        # print(local_model)
        return local_model