# -*- coding: utf-8 -*-
#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#


import os
import dbus
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import  QWidget
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from MeetingTask import MeetingTask
from bubble_message import BubbleMessage, MessageType
from MeetingFile import MeetingFile
from PublicTypes import PublicTypes
from Config import Config

public_types = PublicTypes()

class MeetingBottomWidget(QWidget):
        switch_signal = pyqtSignal(bool)

        def __init__(self):
                super().__init__()
                self.setObjectName('meetingBottomWidget')
                self.setStyleSheet(
                       '''
                        #meetingBottomWidget{
                                border: 0px;
                        }
                        '''
                )

                self.setWindowFlags(Qt.FramelessWindowHint)
                self.ui = Ui_meetingBottomWidget()
                self.ui.setupUi(self)
                self.setContextMenuPolicy(Qt.NoContextMenu)
                self.meetingStatus = False
                self.ui.meetingSwitchBtn.clicked.connect(self.meeting_switch_handel)
                self.ui.exportMeetingBtn.clicked.connect(self.open_meeting_file)


        # def paintEvent(self, event):
        #         painter = QPainter(self)
        #         # print(self.size())
        #         painter.setOpacity(0.80)
        #         painter.setRenderHint(QPainter.Antialiasing)  # 设置抗锯齿，让圆角更加平滑
        #         # # 定义画笔和填充
        #         painter.setBrush(QBrush(QColor(78,74,121)))  # 画刷颜色设置为白色
        #         painter.setPen(QPen(QColor(38, 38, 38), 0))  # 画笔颜色和粗细
        #         # # 绘制圆角矩形
        #         painter.drawRoundedRect(self.rect(), 10, 10)  # 设置边角为10像素的圆度

        def meeting_switch_handel(self):
            print("meeting_switch_handel")
            if self.meetingStatus:
                self.ui.meetingSwitchBtn.setText("开始会议")
                self.meetingStatus = False
                self.switch_signal.emit(self.meetingStatus)
                # self.meetingTask.stop_task()
            else:
                self.ui.meetingSwitchBtn.setText("结束会议")
                self.meetingStatus = True
                self.switch_signal.emit(self.meetingStatus)
                MeetingFile.crate_temp_file()
                # self.meetingTask.start()
                # self.meetingTask.start_task()

        def open_meeting_file(self):
            default_dir = MeetingFile.get_default_dir()
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            fileDialog = QFileDialog(self)
            fileDialog.setDirectory(default_dir)

            filename, _ = fileDialog.getOpenFileName(self, "打开会议纪要文件", "", "All Files(*);;Text Files(*.txt)",
                                                     options=options)
            # print(filename)
            if  os.path.exists(filename):
                MeetingFile.open_file(filename)
                
        def switchViewType(self):
            if PublicTypes.viewType == "sidebar":
                self.ui.meetingSwitchBtn.setGeometry(183, 74, 114, 36)
                self.ui.exportMeetingBtn.setGeometry(10, 140, 114, 36)
            elif PublicTypes.viewType == "windows":
                self.ui.meetingSwitchBtn.setGeometry(550, 34, 114, 36)
                self.ui.exportMeetingBtn.setGeometry(10, 80, 114, 36)
            else:
                raise NotImplementedError("illegal type")

            

class Ui_meetingBottomWidget(QWidget):
    def setupUi(self, bottomWidget):
        bottomWidget.setObjectName("meetingBottomWidget")
        bottomWidget.resize(513, 84)
        bottomWidget.setMinimumSize(QtCore.QSize(384, 180))
        bottomWidget.setStyleSheet("#meetingBottomWidget {\n"
                                    "background: transparent;\n"
                                    "}")

        self.meetingSwitchBtn = QtWidgets.QPushButton(bottomWidget)
        self.meetingSwitchBtn.setMinimumSize(QtCore.QSize(114, 36))
        self.meetingSwitchBtn.setMaximumSize(QtCore.QSize(114, 36))
        # self.meetingSwitchBtn.setFlat(True)
        self.meetingSwitchBtn.setObjectName("sendBtn")
        self.meetingSwitchBtn.setStyleSheet("QPushButton {\n"
                                   "    background-color: #005dff;\n"
                                            "color: #FFFFFF;\n"
                                            "border:0px;\n"
                                            "border-radius:8px;\n"
                                   "}")
        self.meetingSwitchBtn.setGeometry(183, 74, 114, 36)
        self.meetingSwitchBtn.setText("开始会议")

        self.exportMeetingBtn = QtWidgets.QPushButton(bottomWidget)
        self.exportMeetingBtn.setMinimumSize(QtCore.QSize(114, 36))
        self.exportMeetingBtn.setMaximumSize(QtCore.QSize(114, 36))
        # self.meetingSwitchBtn.setFlat(True)
        self.exportMeetingBtn.setObjectName("sendBtn")
        self.exportMeetingBtn.setStyleSheet("QPushButton {\n"
                                            "    background-color: #005dff;\n"
                                            "    background-color: rgba(0,95,255, 80);\n"
                                            "color: #FFFFFF;\n"
                                            "border:0px;\n"
                                            "border-radius:8px;\n"
                                            "}")
        self.exportMeetingBtn.setGeometry(10, 140, 114, 36)
        self.exportMeetingBtn.setText("查看会议纪要")

        self.retranslateUi(bottomWidget)
        QtCore.QMetaObject.connectSlotsByName(bottomWidget)

    def retranslateUi(self, bottomWidget):
        _translate = QtCore.QCoreApplication.translate
        bottomWidget.setWindowTitle(_translate("meetingBottomWidget", "Form"))


# #  测试布局和交互效果
# from PyQt5.QtWidgets import *
# from PyQt5.QtCore import *
# from PyQt5.QtGui import *
#
# if __name__ == '__main__':
#     app = QApplication([])
#     widget = MeetingBottomWidget()
#     widget.show()
#     app.exec_()
