#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import os

from PyQt5.QtCore import QThread, pyqtSignal
import dbus
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib
import subprocess
from Config import Config
from MeetingFile import MeetingFile

def signal_handler(content):
    print("revice_callback:" + content)

class MeetingTask(QThread):
    message_ready_signal = pyqtSignal(str)
    meeting_status_signal = pyqtSignal(str)

    def __init__(self):
        super(MeetingTask, self).__init__()
        self.status = ""
        DBusGMainLoop(set_as_default=True)
        self.bus = dbus.SessionBus()  # 或 dbus.SessionBus()，取决于你的服务在哪里运行

    def message_signal_handler(self, message):
        self.message_ready_signal.emit(message)
        print("message_signal_handler:" + message)

    def status_signal_handler(self, message):
        # self.meeting_status_signal.emit(message)
        if message == "AccessKeyId is mandatory for this action.":
            message = "云端AI模型接口连接失败，请检查设置信息后重试。"
            print("status_signal_handler:" + message)

    def stop(self):
        print("stop")

    def close_bin_process(self):
        # process = subprocess.Popen(["killall", "RealTimeMeeting"])
        os.system("killall RealTimeMeeting")

    def exec_bin_process(self):
        # process = subprocess.Popen(["/bin/sh", "-c", "/usr/share/aiassistant/RealTimeMeeting"])
        process = subprocess.Popen(["/usr/share/aiassistant/bin/RealTimeMeeting"])

    def meeting_init(self):
        bus = dbus.SessionBus()
        meetingObj = bus.get_object('com.nfs.aiassistant', '/')
        meetingInterface = dbus.Interface(meetingObj, 'com.nfs.aiassistant.meetinginterface')
        meetingInterface.Init(Config.get_tongyi_app_key(), Config.get_tongyi_access_key_id(), Config.get_tongyi_access_key_secret())

    def start_task(self):
        self.meeting_init()
        bus = dbus.SessionBus()
        meetingObj = bus.get_object('com.nfs.aiassistant', '/')
        meetingInterface = dbus.Interface(meetingObj, 'com.nfs.aiassistant.meetinginterface')
        meetingInterface.StartMeetingService()
        result = meetingInterface.GetTaskStatus()
        if not result:
            message = "云端AI模型接口连接失败，请检查设置信息后重试。"
            self.message_ready_signal.emit(message)

    def stop_task(self):
        bus = dbus.SessionBus()
        meetingObj = bus.get_object('com.nfs.aiassistant', '/')
        meetingInterface = dbus.Interface(meetingObj, 'com.nfs.aiassistant.meetinginterface')
        meetingInterface.StopMeetingService()
        self.close_bin_process()
        # 获取会议纪要文件
        meeting_file = MeetingFile.get_current_meeting_file()
        # 判断文件是否为空
        if os.path.exists(meeting_file):
            message = "您已结束会议，会议纪要文件" + meeting_file + "已生成，详情请点击下方按钮查看。"
            self.message_ready_signal.emit(message)


    def run(self):

        # 假设服务的名称是 org.freedesktop.DBus.ExampleService，对象路径是 /org/freedesktop/DBus/ExampleObject，接口是 org.freedesktop.DBus.Example
        proxy_obj = self.bus.get_object('com.nfs.aiassistant', '/')
        iface = dbus.Interface(proxy_obj, 'com.nfs.aiassistant.meetinginterface')

        # 连接信号到处理函数
        # iface.connect_to_signal("MessageReady", signal_handler)

        self.bus.add_signal_receiver(self.message_signal_handler, signal_name='MessageReady',
                                dbus_interface='com.nfs.aiassistant.meetinginterface',
                                bus_name='com.nfs.aiassistant', path='/')

        self.bus.add_signal_receiver(self.status_signal_handler, signal_name='ServiceStatus',
                                dbus_interface='com.nfs.aiassistant.meetinginterface',
                                bus_name='com.nfs.aiassistant', path='/')

        # 开始主循环
        self.mainloop = GLib.MainLoop()
        self.mainloop.run()
