#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import dbus
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib
from PyQt5.QtCore import QThread, pyqtSignal
import time
loop = None



class PluginsInstallTask(QThread):
    process_signal = pyqtSignal(str)

    def __init__(self):
        super(PluginsInstallTask, self).__init__()
        self.keep_running = True
        self.packages = None
        self.transaction_interface = None
        self.loop = None
    def set_packages(self, pkgs):
        # self.packages = ['wps-office']
        self.packages = pkgs



    # 等待安装完成
    def on_finished(self, exit_state):
        print(f'Installation finished with exit state: {exit_state}')
        self.loop.quit()

    def on_cancel(self):
        self.loop.quit()

    # 监听属性变化信号
    def on_properties_changed(interface_property, value):
        print(f"on_properties_changed: {interface_property}:{value}")
        if 'Progress' == interface_property:
            print(f'Installation progress: {value}%')
        if 'Download' == interface_property:
            print(f'Download: {value}')
        if 'Status' == interface_property:
            print(f'Status: {value}')

    def install_pkg(self):
        # 连接到系统总线
        bus = dbus.SystemBus()

        # 获取aptdaemon的DBus对象
        aptdaemon_object = bus.get_object('org.debian.apt', '/org/debian/apt')

        # 获取aptdaemon的接口
        aptdaemon_interface = dbus.Interface(aptdaemon_object, 'org.debian.apt')

        # 调用安装包的方法
        transaction_path = aptdaemon_interface.InstallPackages(self.packages)

        # 获取事务对象
        transaction_object = bus.get_object('org.debian.apt', transaction_path)
        self.transaction_interface = dbus.Interface(transaction_object, 'org.debian.apt.transaction')

        self.transaction_interface.connect_to_signal('PropertyChanged', self.on_properties_changed)

        # 连接到完成信号
        self.transaction_interface.connect_to_signal('Finished', self.on_finished)
        self.transaction_interface.Run()

    def run(self):
        try:

            self.install_pkg()

        except Exception as e:
            print(f"组件安装输出任务异常: {e}")
