#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtNetwork import *
from mainwin import ShadowWindow
import sys
from Config import Config
import os
 
class SingleApplication(QApplication):
    def __init__(self, argv):
        QApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
        QApplication.__init__(self, argv)
        self.main_window = None
        self.local_server = None
        self.is_running = False
        self.tray_icon = None
        # 取应用程序名作为LocalServer的名字，也可以自己取一个str，不要和别的软件重复
        # self.server_name = QFileInfo(QCoreApplication.applicationFilePath()).fileName()
        self.server_name = "aiassistant"
        self.init_local_connection()
        self.create_tray_icon()
 
    def init_local_connection(self):
        socket = QLocalSocket()
        socket.connectToServer(self.server_name)
        if socket.waitForConnected(500):
            self.is_running = True
        else:
            self.is_running = False
            self.local_server = QLocalServer()
            self.local_server.newConnection.connect(self.new_local_connection)
            # 监听，如果监听失败，可能是之前程序崩溃时残留进程服务导致的，移除残留进程
            if not self.local_server.listen(self.server_name):
                if self.local_server.serverError() == QAbstractSocket.AddressInUseError:
                    QLocalServer.removeServer(self.server_name)
                    self.local_server.listen(self.server_name)
 
    def new_local_connection(self):
        client_connection = self.local_server.nextPendingConnection()
        client_connection.readyRead.connect(self.handle_local_message)

    def handle_local_message(self):
        print("handle_local_message")
        #无论接受到什么消息，都显示主窗口
        if self.main_window is not None:
            self.main_window.showNormal()
            self.main_window.raise_()
            self.main_window.activateWindow()

    def create_tray_icon(self):
        # 创建图标
        self.tray_icon = QSystemTrayIcon(QIcon("./data/icon.png"), self)

        # 右键菜单栏
        tray_menu = QMenu()

        open_action = QAction("打开方德AI助手", self)
        exit_action = QAction("退出", self)

        tray_menu.addAction(open_action)
        tray_menu.addAction(exit_action)

        self.tray_icon.setContextMenu(tray_menu)

        # 连接事件
        open_action.triggered.connect(self.show_main_window)
        exit_action.triggered.connect(self.quit_application)

        # 连接事件
        self.tray_icon.activated.connect(self.tray_icon_clicked)

        self.tray_icon.show()

    def tray_icon_clicked(self, reason):
        if reason == QSystemTrayIcon.Trigger:  # 左键
            if self.main_window.window_visible:
                self.main_window.hide()
            else:
                self.show_main_window()
            self.main_window.window_visible = not self.main_window.window_visible

    def show_main_window(self):
        if self.main_window is None:
            self.main_window = ShadowWindow()
        self.main_window.show()
        self.main_window.raise_()
        self.main_window.activateWindow()

    def quit_application(self):
        self.tray_icon.hide()
        QApplication.quit()

if __name__ == '__main__':
    app = SingleApplication(sys.argv)
    if not app.is_running:
        os.environ['ALIBABA_CLOUD_ACCESS_KEY_ID'] = Config.get_tongyi_access_key_id()
        os.environ['ALIBABA_CLOUD_ACCESS_KEY_SECRET'] = Config.get_tongyi_access_key_secret()
        app.main_window = ShadowWindow()
        app.main_window.window_visible = True
        app.main_window.show()
        # app.main_window.show()
        sys.exit(app.exec_())
            
# if __name__ == '__main__':
#     app = SingleApplication(sys.argv)
#     if not app.is_running:
#         the_window = ShadowWindow()
#         app.main_window = the_window
#         app.main_window.show()
#         # 其他托盘等代码
#         sys.exit(app.exec_())
#     else:
#         # 新实例启动时向现有实例发送消息
#         socket = QLocalSocket()
#         socket.connectToServer(app.server_name)
#         if socket.waitForConnected():
#             data = "show_window"
#             encode_data = data.encode('utf-8')
#             socket.write(encode_data)  # 发送显示窗口的请求
#             socket.flush()
#             socket.waitForBytesWritten()
#             socket.disconnectFromServer()
#         sys.exit()          