# -*- coding: utf-8 -*-
#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#



from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import  QWidget
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtGui import QPixmap, QIcon
from ChatPlainTextEdit import ChatPlainTextEdit
from enum import Enum
from PublicTypes import PublicTypes
from preGuidWidget import ModelType
from Config import Config

class BottomWidget(QWidget):
        collect_voice_signal = pyqtSignal()
        collect_voice_flag = False
        send_text_signal = pyqtSignal(str)

        def __init__(self):
                super().__init__()
                self.setObjectName('bottomWidget')
                self.setStyleSheet(
                       '''
                        #bottomWidget{
                                border: 0px;
                        }
                        '''
                )

                self.setWindowFlags(Qt.FramelessWindowHint)
                self.ui = Ui_bottomWidget()
                self.ui.setupUi(self)
                self.setContextMenuPolicy(Qt.NoContextMenu)

                self.init_connect()
                self.setPlainEditStyle()
                self.setQPlainTextEditStyle()

                self.current_image_index = 0
                self.total_images =23  # 图片总数-1

                self.timer = QTimer(self)  # 定义定时器
                self.timer.timeout.connect(self.updateImage)  # 定时器信号连接到updateImage方法
                # self.timer.start(60)

        def updateImage(self):
                if self.current_image_index > self.total_images:
                        self.current_image_index = 0  # 重置索引

                image_path = f"./data/audio_input_loading/{self.current_image_index:05d}.png"
                pixmap = QPixmap(image_path)

                if not pixmap.isNull():
                # self.label.setPixmap(pixmap)
                        self.ui.audioInputBtn.setIcon(QIcon(image_path))
                        self.ui.audioInputBtn.setIconSize(QSize(24, 24)); 
                        self.current_image_index += 1
                else:
                        print(f"图片加载失败: {image_path}")
                        self.current_image_index += 1                


        def setPlainEditStyle(self):
                self.ui.inputLineEdit.setStyleSheet(
                '''
                QScrollBar:vertical {
                border-width: 0px;
                border: none;
                background:rgba(0, 0, 0, 0);
                width:5px;
                margin: 0px 0px 0px 0px;
                }
                QScrollBar::handle:vertical {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                stop: 0 #DDDDDD, stop: 0.5 #DDDDDD, stop:1 #aaaaff);
                min-height: 20px;
                max-height: 20px;
                margin: 0 0px 0 0px;
                border-radius: 2px;
                }
                QScrollBar::add-line:vertical {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                stop: 0 rgba(64, 65, 79, 0), stop: 0.5 rgba(64, 65, 79, 0),  stop:1 rgba(64, 65, 79, 0));
                height: 0px;
                border: none;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
                }
                QScrollBar::sub-line:vertical {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                stop: 0  rgba(64, 65, 79, 0), stop: 0.5 rgba(64, 65, 79, 0),  stop:1 rgba(64, 65, 79, 0));
                height: 0 px;
                border: none;
                subcontrol-position: top;
                subcontrol-origin: margin;
                }
                QScrollBar::sub-page:vertical {
                background: rgba(64, 65, 79, 0);
                }

                QScrollBar::add-page:vertical {
                background: rgba(64, 65, 79, 0);
                }

                QPlainTextEdit {
                        background:transparent;
                        color:#333333;
                        font: 14px "Sans Serif";
                }
                '''
                )

        def setQPlainTextEditStyle(self):
            # 根据窗口类型设置不同的文字颜色样式
            if PublicTypes.viewType == "sidebar":
                color_style = "color: white;"
            else:
                color_style = "color: #333333;"

            self.ui.inputLineEdit.setStyleSheet(
                f'''
                QPlainTextEdit {{
                    {color_style}
                    background: transparent;
                    font: 14px "Sans Serif";
                }}
                '''
            )

        def paintEvent(self, event):
                painter = QPainter(self)
                # print(self.size())
                painter.setOpacity(0.80)
                painter.setRenderHint(QPainter.Antialiasing)  # 设置抗锯齿，让圆角更加平滑
                # # 定义画笔和填充
                if PublicTypes.viewType =="sidebar":
                    painter.setBrush(QBrush(QColor(48, 66, 92)))  # 画刷颜色设置为白色
                    painter.setPen(QPen(QColor(48, 66, 92), 0))  # 画笔颜色和粗细
                else:
                    painter.setBrush(QBrush(QColor(189,195,209)))  # 画刷颜色设置为白色
                    painter.setPen(QPen(QColor(189,195,209), 0))  # 画笔颜色和粗细
                # # 绘制圆角矩形
                painter.drawRoundedRect(self.rect(), 10, 10)  # 设置边角为10像素的圆度


        def init_connect(self):
            self.ui.sendBtn.clicked.connect(self.send_text_message)
            self.ui.audioInputBtn.clicked.connect(self.capture_voice)
            self.ui.inputLineEdit.return_signal.connect(self.return_signal_handle)

        def slot_shift_text_input(self):
            print('shiftToTextInput')
        #     self.ui.tabWidget.setCurrentIndex(1)
            # self.ui.tabWidget.setFixedHeight(80)

        def slot_shift_audio_input(self):
            print('shiftToAudioInput')
        #     self.ui.tabWidget.setCurrentIndex(0)
            # self.ui.tabWidget.setFixedHeight(147)

        def start_timer(self):
                self.timer.start(60)
                self.ui.inputLineEdit.show()
                # self.ui.audioInputLabel.setPlainText("语音采集中")

        def stop_timer(self):
                self.timer.stop()
                icon = QtGui.QIcon()

                if PublicTypes.viewType =="sidebar":
                    icon.addPixmap(QtGui.QPixmap("./data/microphone_input_sidebar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
                else:
                    icon.addPixmap(QtGui.QPixmap("./data/microphone_input.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

                self.ui.audioInputBtn.setIcon(icon)
                # self.ui.inputLineEdit.hide()
                # self.ui.audioInputLabel.setPlainText("点击讲话")

        def show_collect_content(self, message):
                self.ui.inputLineEdit.setPlainText(message)

        def capture_voice(self):
            print("capture_voice")
            self.collect_voice_signal.emit()

        def shift_model(self):
            print("shift_model")
        #     pos = self.ui.btnShiftModel.mapToGlobal(QPoint(0, 0))- QPoint(self.menu.width(), self.menu.height())
        #     self.menu.move(pos)
        #     self.menu.show()

        def send_message(self, message):
            self.send_text_signal.emit(message)

        # def set_send_button_status(self, status):
        #     self.ui.set_send_button_status(status)

        def set_send_button_status(self, status):
            self.ui.sendBtn.setEnabled(status)

        def return_signal_handle(self):
            if self.ui.sendBtn.isEnabled():
                self.send_text_message()
            else:
                print("sendBtn is disable")

        def send_text_message(self):
            print("send_text_message")

            message = self.ui.inputLineEdit.toPlainText()
            print(message)
            if len(message) == 0 or not self.ui.sendBtn.isEnabled():
                return

            self.ui.sendBtn.setEnabled(False)
            self.send_text_signal.emit(message)
            self.ui.inputLineEdit.setPlainText("")


class Ui_bottomWidget(QWidget):
    def setupUi(self, bottomWidget):
        bottomWidget.setObjectName("bottomWidget")
        bottomWidget.resize(513, 180)
        # bottomWidget.setMinimumSize(QtCore.QSize(384, 160))
        bottomWidget.setStyleSheet("#bottomWidget {\n"
        "    background-color: rgba(48,66,92, 204);\n"
        "}")

        self.verticalLayout = QtWidgets.QVBoxLayout(bottomWidget)
        # self.verticalLayout.setContentsMargins(24, 24, 26 ,12)
        self.verticalLayout.setObjectName("verticalLayout")
        self.inputLineEdit = ChatPlainTextEdit(bottomWidget)
        self.inputLineEdit.setStyleSheet("font: 14px \"Sans Serif\";\n"
"color:white;\n"
"background:transparent;")
        self.inputLineEdit.setObjectName("inputLineEdit")
        self.verticalLayout.addWidget(self.inputLineEdit)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, 12, -1)
        self.horizontalLayout.setSpacing(19)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.audioInputBtn = QtWidgets.QPushButton(bottomWidget)
        self.audioInputBtn.setFixedSize(QtCore.QSize(24, 24))
        self.audioInputBtn.setStyleSheet("border: 0px;")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("./data/microphone_input_sidebar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.audioInputBtn.setIcon(icon)
        self.audioInputBtn.setIconSize(QtCore.QSize(24, 24))
        self.audioInputBtn.setFlat(True)
        self.audioInputBtn.setObjectName("audioInputBtn")
        self.horizontalLayout.addWidget(self.audioInputBtn)
        self.sendBtn = QtWidgets.QPushButton(bottomWidget)
        self.sendBtn.setMinimumSize(QtCore.QSize(24, 24))
        self.sendBtn.setMaximumSize(QtCore.QSize(24, 24))
        self.sendBtn.setStyleSheet("border:0px;")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("./data/send_sidebar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.sendBtn.setIcon(icon1)
        self.sendBtn.setIconSize(QtCore.QSize(24, 24))
        self.sendBtn.setFlat(True)
        self.sendBtn.setObjectName("sendBtn")
        self.horizontalLayout.addWidget(self.sendBtn)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(bottomWidget)
        QtCore.QMetaObject.connectSlotsByName(bottomWidget)

    def retranslateUi(self, bottomWidget):
        _translate = QtCore.QCoreApplication.translate
        bottomWidget.setWindowTitle(_translate("bottomWidget", "Form"))

# class ModelEnum(Enum):
#     DeepSeek = 1
#     Ali = 2
#     Ifly = 3
class ModelMenu(QMenu):
    def __init__(self, *args, **kwargs):
        super(ModelMenu, self).__init__()
        self.radius = 10
        self.setting_menu_sytle =  '''
            QMenu {{
                /* 半透明效果 */
                border-radius: {radius};
                border: 2px solid rgb(48,66,92);
                background-color: rgba(48,66,92, 204);
                color:white;
            }}
            QMenu::item {{
                border-radius: {radius};
                /* 这个距离很麻烦需要根据菜单的长度和图标等因素微调 */
                padding: 8px 48px 8px 12px; /* 12px是文字距离左侧距离*/
                background-color: transparent;
            }}

            /* 鼠标悬停和按下效果 */
            QMenu::item:selected {{
                border-radius: {radius};
                /* 半透明效果 */
                background-color: rgba(200, 210, 220, 232);
                color: black;
            }}

            /* 禁用效果 */
            QMenu::item:disabled {{
                border-radius: {radius};
                background-color: transparent;
            }}

            /* 图标距离左侧距离 */
            QMenu::icon {{
                left: 15px;
            }}

            /* 分割线效果 */
            QMenu::separator {{
                height: 1px;
                background-color: rgb(232, 236, 243);
            }}'''.format(radius=self.radius)
        self.setStyleSheet(self.setting_menu_sytle)
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setWindowFlags(self.windowFlags()  | Qt.FramelessWindowHint )

class BottomUi(QWidget):
    def __init__(self):
        super().__init__()
        self.setObjectName('bottomUi')
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        # self.setMinimumHeight(400)
        self.bottomContent = BottomWidget()
        # self.bottomContent.setMaximumHeight(120)
        self.verticalLayout = QtWidgets.QVBoxLayout(self)
        self.verticalLayout.setSpacing(8)
        self.modeType = ModelType.Chat
        self.createShiftModel()
        self.verticalLayout.addWidget(self.bottomContent)


    def createShiftModel(self):
        self.aliAction = QAction(QIcon('./data/_icon_ali.svg'), '阿里通义', self)
        self.iFlyAction = QAction(QIcon('./data/_icon_ifly.svg'), '讯飞星火', self)
        self.deepseekAction = QAction(QIcon('./data/_icon_deekseek.svg'), 'DeepSeek-R1', self)
        self.deepseekLocalAction = QAction(QIcon('./data/_icon_deekseek.svg'), 'DeepSeek-R1-离线', self)
        self.menu = ModelMenu(self)
        self.deepseekAction.triggered.connect(self.useDeepSeek)
        self.deepseekLocalAction.triggered.connect(self.useDeepSeekLocal)
        self.aliAction.triggered.connect(self.useAliTY)
        self.iFlyAction.triggered.connect(self.useIFly)

        self.menu.addAction(self.deepseekAction)
        if Config.get_ds_local_model():
            self.menu.addAction(self.deepseekLocalAction)
        self.menu.addAction(self.aliAction)
        self.menu.addAction(self.iFlyAction)

        self.setWindowFlags(Qt.Popup | Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        self.expendStr = "\u2228"  #向下符号，展开
        self.modelBtn = QPushButton()
        self.modelBtn.setFixedSize(150, 30)
        self.modelBtn.setStyleSheet("""
                background-color:  rgba(48,66,92, 204);
                border: 1px solid rgb(125, 125, 125); /*整个菜单区域的边框粗细、样式、颜色*/
                border-radius: 5px;
                color: white;
            """)
        self.modelBtn.clicked.connect(self.on_button_click)
        self.useIFly()

        spaceItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        hLayout = QtWidgets.QHBoxLayout()
        hLayout.addItem(spaceItem)
        hLayout.addWidget(self.modelBtn)

        self.verticalLayout.addLayout(hLayout)

    def updateMenu(self, modeType):
        print(modeType)
        if modeType != self.modeType:
            if self.modeType == ModelType.Chat:
                for action in self.menu.actions():
                    if action.text() == "DeepSeek-R1":
                        if self.modelBtn.text().find("DeepSeek-R1") != -1:
                            self.useIFly()
                        self.menu.removeAction(self.deepseekAction)
                    if action.text() == "DeepSeek-R1-离线":
                        if self.modelBtn.text().find("DeepSeek-R1-离线") != -1:
                            self.useIFly()
                        self.menu.removeAction(self.deepseekLocalAction)

                self.modeType = modeType
            else:
                for action in self.menu.actions():
                    if action.text() != "DeepSeek-R1":
                        self.menu.addAction(self.deepseekAction)
                        if Config.get_ds_local_model():
                            self.menu.addAction(self.deepseekLocalAction)

                self.modeType = modeType


    def on_button_click(self):
        print("on_button_click")
        buttonGlobalPos = self.modelBtn.mapToGlobal(self.modelBtn.pos())
        buttonHeight = self.modelBtn.height()
        menuHeight = self.menu.sizeHint().height()
        menuY = buttonGlobalPos.y() - menuHeight - self.modelBtn.pos().y()
        menuX = buttonGlobalPos.x() - self.modelBtn.pos().x() - self.menu.sizeHint().width() + self.modelBtn.width()
        print(menuX)
        print(menuX)
        menuPos = QPoint(menuX, menuY)
        self.menu.exec(menuPos)

    def useDeepSeek(self):
        print("Use deep seek")
        icon = self.deepseekAction.icon()
        txt = self.deepseekAction.text()

        self.modelBtn.setIcon(icon)
        self.modelBtn.setText(f'{txt}  \u2228')
        # self.model = ModelEnum.DeepSeek
        if self.modeType == ModelType.Chat:
            Config.set_chat_model_name("DEEPSEEK")

    def useDeepSeekLocal(self):
        print("Use deep seek local")
        icon = self.deepseekLocalAction.icon()
        txt = self.deepseekLocalAction.text()

        self.modelBtn.setIcon(icon)
        self.modelBtn.setText(f'{txt}  \u2228')
        # self.model = ModelEnum.DeepSeek
        if self.modeType == ModelType.Chat:
            Config.set_chat_model_name("DEEPSEEKLOCAL")

    def useAliTY(self):
        print("Use ali tongyi")
        icon = self.aliAction.icon()
        txt = self.aliAction.text()

        self.modelBtn.setIcon(icon)
        self.modelBtn.setText(f'{txt}  \u2228')
        # self.model = ModelEnum.Ali
        if self.modeType == ModelType.Chat:
            Config.set_chat_model_name("TONGYI")
        elif self.modeType == ModelType.Paint:
            Config.set_drawing_model_name("TONGYI")
        else:
            print("")

    def useIFly(self):
        print("Use ifly")
        icon = self.iFlyAction.icon()
        txt = self.iFlyAction.text()

        self.modelBtn.setIcon(icon)
        self.modelBtn.setText(f'{txt}  \u2228')
        # self.model = ModelEnum.Ifly
        if self.modeType == ModelType.Chat:
            Config.set_chat_model_name("XINGHUO")
        elif self.modeType == ModelType.Paint:
            Config.set_drawing_model_name("XINGHUO")
        else:
            print("")

# #  测试布局和交互效果
# from PyQt5.QtWidgets import *
# from PyQt5.QtCore import *
# from PyQt5.QtGui import *
#
# if __name__ == '__main__':
#     app = QApplication([])
#     widget = BottomWidget()
#     widget.show()
#     app.exec_()
