#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

from PIL import Image
from PyQt5 import QtGui
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PublicTypes import PublicTypes
from Config import Config

class MessageType:
    Text = 1
    Image = 2
    Guid = 3
    Waiting = 4

class TextMessage(QLabel):
    heightSingal = pyqtSignal(int)

    def __init__(self, text, is_send=False, parent=None):
        super(TextMessage, self).__init__(text, parent)
        font = QFont('微软雅黑', 12)
        self.setFont(font)
        self.setWordWrap(True)
        self.setMaximumWidth(450)
        self.setMinimumWidth(90)
        self.setMinimumHeight(40)
        self.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.setContextMenuPolicy(Qt.NoContextMenu)
        self.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
        if is_send:
            self.setAlignment(Qt.AlignLeft)
            self.setStyleSheet(
                '''
                background-color: rgba(0,93,255,204);
                border-radius:10px;
                padding:10px;
                color: white;
                '''
            )
        else:
            self.setAlignment(Qt.AlignLeft)
            if PublicTypes.viewType=="sidebar":
                self.setStyleSheet(
                    '''
                    background-color: rgba(54,54,54,204);
                    border-radius:10px;
                    padding:10px;
                    color: white;
                    '''
                )
            if PublicTypes.viewType=="window":
                self.setStyleSheet(
                    '''
                    background-color: rgba(204,204,204,204);
                    border-radius:10px;
                    padding:10px;
                    color: black;
                    '''
                )

        self.font_metrics = QFontMetrics(font)
        rect = self.font_metrics.boundingRect(0,0,480,1000,Qt.TextWordWrap, text)
        # 调整窗口尺寸以匹配计算出的文本区域尺寸
        self.setMinimumHeight(rect.height() + 20)  # 这里加20作为额外的垂直边距
        self.setMaximumWidth(rect.width())   #修复文本不合理换行问题
        self.adjustSize()

    def meeting_content_adjust_size(self):
        self.setMaximumWidth(500)
        # self.setFixedWidth(480)
        self.adjustSize()

    def adjust_size(self, text):
        rect = self.font_metrics.boundingRect(0, 0, 480, 1000, Qt.TextWordWrap, text)
        # 调整窗口尺寸以匹配计算出的文本区域尺寸
        self.setMinimumHeight(rect.height() + 20)  # 这里加20作为额外的垂直边距
        self.setMaximumWidth(rect.width())  # 修复文本不合理换行问题
        self.adjustSize()

    def paintEvent(self, a0: QtGui.QPaintEvent) -> None:
        super(TextMessage, self).paintEvent(a0)

class Notice(QLabel):
    def __init__(self, text, type_=3, parent=None):
        super().__init__(text, parent)
        self.type_ = type_
        self.setFont(QFont('微软雅黑', 12))
        self.setWordWrap(True)
        self.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.setAlignment(Qt.AlignCenter)
        self.setStyleSheet(
            '''
            color: white;
            '''
        )

class Avatar(QLabel):
    def __init__(self, avatar, parent=None):
        super().__init__(parent)
        if isinstance(avatar, str):
            self.setPixmap(QPixmap(avatar).scaled(45, 45))
            self.image_path = avatar
        elif isinstance(avatar, QPixmap):
            self.setPixmap(avatar.scaled(45, 45))
        self.setFixedSize(QSize(45, 45))


class OpenImageThread(QThread):
    def __init__(self, image_path):
        super().__init__()
        self.image_path = image_path

    def run(self) -> None:
        image = Image.open(self.image_path)
        image.show()


class ImageMessage(QLabel):
    def __init__(self, avatar, parent=None):
        super().__init__(parent)
        self.image = QLabel(self)
        if isinstance(avatar, str):
            self.setPixmap(QPixmap(avatar))
            self.image_path = avatar
        elif isinstance(avatar, QPixmap):
            self.setPixmap(avatar)
        self.setMaximumWidth(420)
        self.setMaximumHeight(420)
        self.setScaledContents(True)
        
    def mousePressEvent(self, event):
        if event.buttons() == Qt.LeftButton:  # 左键按下
            self.open_image_thread = OpenImageThread(self.image_path)
            self.open_image_thread.start()

class WaitingMessage(QLabel):
    def __init__(self, parent=None):      
        super().__init__(parent)
        self.setFixedSize(QSize(95, 36))
        self.setStyleSheet(
            '''
            background-color: rgba(235,235,235,204);
            border-radius:10px;
            padding:10px;
            color: white;
            '''
        )

        self.current_image_index = 0
        self.total_images = 39  # 动画图片总数（0~39）
        self.timer = QTimer(self)  # 定义定时器
        self.timer.timeout.connect(self.updateLoadingImage)  # 定时器信号连接到updateImage方法
        self.timer.start(60)

    def updateLoadingImage(self):
            if self.current_image_index > self.total_images:
                    self.current_image_index = 0  # 重置索引

            image_path = f"./data/dialog_loading/{self.current_image_index:05d}.png"
            pixmap = QPixmap(image_path)

            if not pixmap.isNull():
            # self.label.setPixmap(pixmap)
                    self.setPixmap(pixmap)
                    self.current_image_index += 1
            else:
                    print(f"图片加载失败: {image_path}")
                    self.current_image_index += 1                

class GuidMessage(QLabel):
    image_path = ""

    def __init__(self, text, font_size, parent=None):
        super(GuidMessage, self).__init__(text, parent)
        font = QFont('微软雅黑', font_size)
        self.setFont(font)
        self.setWordWrap(True)
        self.setMaximumWidth(400)
        self.setMinimumWidth(100)
        self.setMinimumHeight(45)
        # self.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Ignored)
        self.setWordWrap(True)
        # self.setAlignment(Qt.AlignLeft)
        self.setStyleSheet(
            '''
            background-color:transparent;
            border-radius:10px;
            padding:10px;
            color: #FFFFFF;
            '''
        )

        font_metrics = QFontMetrics(font)
        rect = font_metrics.boundingRect(text)
        self.setMaximumWidth(rect.width()+30)

class BubbleMessage(QWidget):
    switch_signal = pyqtSignal(bool, str, QWidget)
    def __init__(self, str_content, avatar, Type, font_size, is_send=False, parent=None):
        super().__init__(parent)
        self.isSend = is_send
        self.isPlayAudio = False
        if Config.get_text_play_flag() == "True":
            self.isPlayAudio = True
        # self.set
        if PublicTypes.viewType == "sidebar":
            self.setStyleSheet(
                '''
                    background-color: rgba(54,54,54,204);
                    border-radius:10px;
                    padding:10px;
                    color: white;
                '''
            )
        elif PublicTypes.viewType == "windows":
            self.setStyleSheet(
                '''
                    background-color: rgba(204,204,204,204);
                    border-radius:10px;
                    padding:10px;
                    color: black;
                '''
            )

        self.playBtn = QPushButton(self)
        # self.playBtn.setCheckable(True)
        self.playBtn.clicked.connect(self.playAudio)
        self.playBtn.hide()
        self.playBtn.setIconSize(QSize(16, 16))
        self.playBtn.setFixedSize(QSize(16, 16))
        self.playBtn.setStyleSheet(
                '''
                background-color: transparent;
                border:none;
                '''
            )
        self.playBtn.setFlat(True)
        if PublicTypes.viewType == "sidebar":  #默认显示播放
            self.playBtn.setIcon(QIcon('./data/play.png'))
        elif PublicTypes.viewType == "windows":
            self.playBtn.setIcon(QIcon('./data/play_window.png'))
        # self.playBtn.setChecked(True) #默认播放
        layout = QHBoxLayout()
        layout.setSpacing(8)
        layout.setContentsMargins(0, 5, 5, 5)
        if Type == MessageType.Text:
            self.message = TextMessage(str_content, is_send)
            if  self.isPlayAudio:
                if PublicTypes.viewType == "sidebar":
                    self.playBtn.setIcon(QIcon('./data/stop.png'))
                elif PublicTypes.viewType == "windows":
                    self.playBtn.setIcon(QIcon('./data/stop_window.png'))
            # self.message.setMaximumWidth(int(self.width() * 0.6))
        elif Type == MessageType.Image:
            self.message = ImageMessage(str_content)
        elif Type == MessageType.Guid:
            self.message = GuidMessage(str_content, font_size)
        elif Type == MessageType.Waiting:
            self.message = WaitingMessage()
        else:
            raise ValueError("未知的消息类型")
        

        self.spacerItem = QSpacerItem(45 + 6, 45, QSizePolicy.Expanding, QSizePolicy.Minimum)
        if is_send:
            layout.addItem(self.spacerItem)
            layout.addWidget(self.message, 1)
        else:
            layout.addWidget(self.message, 1)
            #如果是AI回复的消息，则显示播放按钮
            if Type == MessageType.Text:
                self.playBtn.show()
                layout.addWidget(self.playBtn, 0, Qt.AlignCenter)

            if Type != MessageType.Guid:
                layout.addItem(self.spacerItem)
        
        self.setLayout(layout)

    def playAudio(self):
        print("play audio", self.message.text())
        if self.isPlayAudio:
            if PublicTypes.viewType == "sidebar":
                self.playBtn.setIcon(QIcon('./data/play.png'))
            elif PublicTypes.viewType == "windows":
                self.playBtn.setIcon(QIcon('./data/play_window.png'))
        else:
            if PublicTypes.viewType == "sidebar":
                self.playBtn.setIcon(QIcon('./data/stop.png'))
            elif PublicTypes.viewType == "windows":
                self.playBtn.setIcon(QIcon('./data/stop_window.png'))
        self.isPlayAudio = not self.isPlayAudio
        self.switch_signal.emit(self.isPlayAudio, self.message.text(), self)

    def playComplete(self):
        print("playComplete")
        if PublicTypes.viewType == "sidebar":
            self.playBtn.setIcon(QIcon('./data/play.png'))
        elif PublicTypes.viewType == "windows":
            self.playBtn.setIcon(QIcon('./data/play_window.png'))
        self.isPlayAudio = False

    def updatePlayIcon(self):
        # 更新 playBtn 图标
        if PublicTypes.viewType == "sidebar":
            if not self.isPlayAudio:
                self.playBtn.setIcon(QIcon('./data/play.png'))
            else:
                self.playBtn.setIcon(QIcon('./data/stop.png'))
        elif PublicTypes.viewType == "windows":
            if not self.isPlayAudio:
                self.playBtn.setIcon(QIcon('./data/play_window.png'))
            else:
                self.playBtn.setIcon(QIcon('./data/stop_window.png'))

class ScrollAreaContent(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.adjustSize()

class ScrollArea(QScrollArea):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWidgetResizable(True)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setStyleSheet(
            '''
            border:none;
            '''
        )


class ScrollBar(QScrollBar):
    def __init__(self):
        super().__init__()
        self.setStyleSheet(
            '''
          QScrollBar:vertical {
              border-width: 0px;
              border: none;
              background:rgba(64, 65, 79, 0);
              width:5px;
              margin: 0px 0px 0px 0px;
          }
          QScrollBar::handle:vertical {
              background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
              stop: 0 #DDDDDD, stop: 0.5 #DDDDDD, stop:1 #aaaaff);
              min-height: 20px;
              max-height: 20px;
              margin: 0 0px 0 0px;
              border-radius: 2px;
          }
          QScrollBar::add-line:vertical {
              background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
              stop: 0 rgba(64, 65, 79, 0), stop: 0.5 rgba(64, 65, 79, 0),  stop:1 rgba(64, 65, 79, 0));
              height: 0px;
              border: none;
              subcontrol-position: bottom;
              subcontrol-origin: margin;
          }
          QScrollBar::sub-line:vertical {
              background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
              stop: 0  rgba(64, 65, 79, 0), stop: 0.5 rgba(64, 65, 79, 0),  stop:1 rgba(64, 65, 79, 0));
              height: 0 px;
              border: none;
              subcontrol-position: top;
              subcontrol-origin: margin;
          }
          QScrollBar::sub-page:vertical {
              background: rgba(64, 65, 79, 0);
          }

          QScrollBar::add-page:vertical {
              background: rgba(64, 65, 79, 0);
          }
            '''
        )


class ChatWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.setMinimumHeight(95)
        self.setMaximumHeight(922)

        self.desktop = QApplication.desktop()
    
        #获取显示器分辨率大小
        self.screenRect = self.desktop.screenGeometry()
        self.height = self.screenRect.height()
  
        print(self.height)
        print(self.width)

        h = self.height - 280
        self.setFixedHeight(h)
        layout = QVBoxLayout()
        layout.setSpacing(0)
        self.adjustSize()
        self.setObjectName('chatwidget')
        # 生成滚动区域
        self.scrollArea = ScrollArea(self)
        scrollBar = ScrollBar()
        self.scrollArea.setVerticalScrollBar(scrollBar)
        self.scrollArea.setFixedHeight(720)
        # self.scrollArea.setGeometry(QRect(9, 9, 261, 211))
        # 生成滚动区域的内容部署层部件
        self.scrollAreaWidgetContents = ScrollAreaContent(self.scrollArea)
        self.scrollAreaWidgetContents.setMinimumSize(50, 100)
        # 设置滚动区域的内容部署部件为前面生成的内容部署层部件
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        layout.addWidget(self.scrollArea)
        self.layout0 = QVBoxLayout()
        self.layout0.setSpacing(0)
        self.scrollAreaWidgetContents.setLayout(self.layout0)
        self.setLayout(layout)

        self.spacerItemAdded = False

        #保持最新消息在当前光标处
        self.scrollArea.verticalScrollBar().rangeChanged.connect(
            lambda: self.scrollArea.verticalScrollBar().setValue(
                self.scrollArea.verticalScrollBar().maximum()
            )
        )

        self.scrollArea.setStyleSheet(
            '''
            QScrollArea {background-color: transparent;}
            '''
        )

        self.scrollArea.viewport().setStyleSheet(
            '''
            background-color: transparent;
            '''
        )

        self.setContextMenuPolicy(Qt.NoContextMenu)

    def add_guid_item(self):
        guid = QLabel()
        guid.setText('我是您的智能助手NFS AI ')
        guid.setFont(QFont('微软雅黑',17))

        content = QLabel()
        content.setText('作为您的智能助手，我将为您提供个性化的操作系统体验,帮助您完成各种任务、回答问题、提供实用信息，以及提供智能建议和提示。')
        content.setFont(QFont('微软雅黑', 14))

        self.layout0.addItem(QSpacerItem(10, 50, QSizePolicy.Preferred, QSizePolicy.Fixed))
        self.layout0.addWidget(guid)
        self.layout0.addWidget(content)
        self.layout0.addItem(QSpacerItem(10, 50, QSizePolicy.Preferred, QSizePolicy.Expanding))

    def clearLayout(self):
            while self.layout0.count():
                item = self.layout0.takeAt(0)
                widget = item.widget()
                if widget:
                    widget.deleteLater()
            self.spacerItemAdded = False

    def add_message_item(self, bubble_message, index=1):
        if index:
            self.layout0.addWidget(bubble_message)
        else:
            item_count = self.layout0.count()
            self.layout0.insertWidget(item_count-1, bubble_message)
        # self.set_scroll_bar_last()
        self.add_vertical_spacer()
            
    def add_vertical_spacer(self):
        if not self.spacerItemAdded:
            self.layout0.addItem(QSpacerItem(10, 100, QSizePolicy.Preferred, QSizePolicy.Expanding))
            self.spacerItemAdded = True

    def remove_message_item(self, bubble_message):
        self.layout0.removeWidget(bubble_message)

    def set_scroll_bar_last(self):
        self.scrollArea.verticalScrollBar().setValue(
            self.scrollArea.verticalScrollBar().maximum()
        )

    def set_scroll_bar_value(self, val):
        self.verticalScrollBar().setValue(val)

    def verticalScrollBar(self):
        return self.scrollArea.verticalScrollBar()

    def update(self) -> None:
        super().update()
        self.scrollAreaWidgetContents.adjustSize()
        self.scrollArea.update()
        
    def switchViewType(self):
        if PublicTypes.viewType == "sidebar":
            h = self.height - 280
            self.setFixedHeight(h)
            self.scrollArea.setFixedHeight(h)
            for i in range(self.layout0.count()):
                wid0 = self.layout0.itemAt(i).widget()
                if isinstance(wid0,BubbleMessage):
                    wid0.updatePlayIcon()
                    if i==0 or i%2==1:
                        wid0.message.setStyleSheet(
                            '''
                            background-color: rgba(54,54,54,204);
                            border-radius:10px;
                            padding:10px;
                            color: white;
                            '''
                        )
                    else:
                        wid0.message.setStyleSheet(
                            '''
                            background-color: rgba(0,93,255,204);
                            border-radius:10px;
                            padding:10px;
                            color: white;
                            '''
                        )
            self.setMaximumWidth(480  - 36)
            self.setFixedWidth(480  - 36)
            self.scrollAreaWidgetContents.setFixedWidth(480  - 36)
            self.scrollArea.setFixedWidth(480  - 36)
            self.update()
        elif PublicTypes.viewType == "windows":
            self.setFixedHeight(500)
            self.scrollArea.setFixedHeight(500)
            for i in range(self.layout0.count()):
                wid0 = self.layout0.itemAt(i).widget()
                if isinstance(wid0,BubbleMessage):
                    wid0.updatePlayIcon()
                    if i==0 or i%2==1:
                        wid0.message.setStyleSheet(
                            '''
                            background-color: rgba(204,204,204,204);
                            border-radius:10px;
                            padding:10px;
                            color: black;
                            '''
                        )
                    else:
                        wid0.message.setStyleSheet(
                            '''
                            background-color: rgba(0,93,255,204);
                            border-radius:10px;
                            padding:10px;
                            color: white;
                            '''
                        )
            self.setFixedWidth(1164)
            self.scrollAreaWidgetContents.setFixedWidth(1164)
            self.scrollArea.setFixedWidth(1164)
            self.update()
        else:
            raise NotImplementedError("illegal type")   


class MeetingWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.setMinimumHeight(95)
        self.setMaximumHeight(922)

        self.desktop = QApplication.desktop()

        # 获取显示器分辨率大小
        self.screenRect = self.desktop.screenGeometry()
        self.height = self.screenRect.height()

        print(self.height)
        print(self.width)

        h = self.height - 280
        self.setFixedHeight(h)
        layout = QVBoxLayout()
        layout.setSpacing(0)
        self.adjustSize()
        self.setObjectName('meetingwidget');
        # 生成滚动区域
        self.scrollArea = ScrollArea(self)
        scrollBar = ScrollBar()
        self.scrollArea.setVerticalScrollBar(scrollBar)
        # self.scrollArea.setGeometry(QRect(9, 9, 261, 211))
        # 生成滚动区域的内容部署层部件
        self.scrollAreaWidgetContents = ScrollAreaContent(self.scrollArea)
        self.scrollAreaWidgetContents.setMinimumSize(50, 100)
        # 设置滚动区域的内容部署部件为前面生成的内容部署层部件
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        
        self.scrollArea.setFixedHeight(720)
        layout.addWidget(self.scrollArea)
        self.layout0 = QVBoxLayout()
        self.layout0.setSpacing(0)
        self.scrollAreaWidgetContents.setLayout(self.layout0)
        self.setLayout(layout)

        self.spacerItemAdded = False

        # 保持最新消息在当前光标处
        self.scrollArea.verticalScrollBar().rangeChanged.connect(
            lambda: self.scrollArea.verticalScrollBar().setValue(
                self.scrollArea.verticalScrollBar().maximum()
            )
        )

        self.scrollArea.setStyleSheet(
            '''
            QScrollArea {background-color: transparent;}
            '''
        )

        self.scrollArea.viewport().setStyleSheet(
            '''
            background-color: transparent;
            '''
        )

        self.setContextMenuPolicy(Qt.NoContextMenu)

    def add_message_item(self, bubble_message, index=1):
        if index:
            self.layout0.addWidget(bubble_message)
        else:
            item_count = self.layout0.count()
            self.layout0.insertWidget(item_count-1, bubble_message)
        # self.set_scroll_bar_last()
        self.add_vertical_spacer()

    def remove_message_item(self, bubble_message):
        self.layout0.removeWidget(bubble_message)

    def add_vertical_spacer(self):
        if not self.spacerItemAdded:
            self.layout0.addItem(QSpacerItem(10, 100, QSizePolicy.Preferred, QSizePolicy.Expanding))
            self.spacerItemAdded = True
            
    def switchViewType(self):
        if PublicTypes.viewType == "sidebar":
            h = self.height - 280
            self.setFixedHeight(h)
            self.scrollArea.setFixedHeight(h)
            self.setFixedWidth(480  - 36)
            self.scrollArea.setFixedWidth(480 - 36)
            self.scrollAreaWidgetContents.setFixedWidth(480 - 36)
            for i in range(self.layout0.count()):
                wid0 = self.layout0.itemAt(i).widget()
                if isinstance(wid0, BubbleMessage):
                    wid0.message.setStyleSheet(
                        '''
                        background-color: rgba(54,54,54,204);
                        border-radius:10px;
                        padding:10px;
                        color: white;
                        '''
                    )
        elif PublicTypes.viewType == "windows":
            self.setFixedHeight(560)
            self.scrollArea.setFixedHeight(560)
            self.setFixedWidth(1164)
            self.scrollArea.setFixedWidth(1164)
            self.scrollAreaWidgetContents.setFixedWidth(1164)
            for i in range(self.layout0.count()):
                wid0 = self.layout0.itemAt(i).widget()
                if isinstance(wid0,BubbleMessage):
                    wid0.message.setStyleSheet(
                        '''
                        background-color: rgba(204,204,204,204);
                        border-radius:10px;
                        padding:10px;
                        color: black;
                        '''
                    )
        else:
            raise NotImplementedError("illegal type")   