# -*- coding: utf-8 -*-
#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import os

import dashscope
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from bubble_message import BubbleMessage, MessageType
from PublicTypes import PublicTypes
from Config import Config




class exChange(QWidget):
    collect_voice_signal = pyqtSignal()
    collect_voice_flag = False
    stop_speech_signal = pyqtSignal()
    stop_speech_flag = False

    def __init__(self):
        super().__init__()
        self.setObjectName('exChangeWindow')
        self.setStyleSheet(
            '''
            #mainwindow{
                border-top-left-radius： 15px;
                border-top-right-radius: 15px;
                border-bottom-left-radius: 15px;
                border-bottom-right-radius: 15px;
            }
            '''
        )

        # 背景透明
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setMaximumHeight(922)
        # 获取显示器分辨率大小
        self.desktop = QApplication.desktop()
        self.screenRect = self.desktop.screenGeometry()
        self.height = self.screenRect.height()
        h = self.height - 380
        self.setFixedHeight(h)


        self.verticalLayout = QtWidgets.QVBoxLayout(self)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(10)

        # AIFace
        self.imageLabel = QLabel(self)
        self.imageLabel.setAlignment(Qt.AlignCenter)
        self.imageLabel.setStyleSheet("QLabel { padding: 0px;}")
        self.imageLabel.setAlignment(Qt.AlignCenter)
        # self.imageLabel.setScaledContents(True)

        # 图片列表和索引
        self.image_folder1 = './data/AIFace'
        self.image_files1 = [os.path.join(self.image_folder1, f) for f in os.listdir(self.image_folder1) if
                             f.endswith(('.png', '.jpg', '.jpeg'))]
        self.current_image_index1 = 0
        self.timecounter=0
        self.image_timer1 = QTimer(self)
        self.image_timer1.timeout.connect(self.updateImage)
        self.image_timer1.start(200)

        self.textLabel = QLabel(self)
        self.textLabel.setText('''
        <p style="font-size: 16px; font-weight: bold;  color: #D3D3D3;padding: 0px; ">欢迎使用方德AI语音助手<p/>
        <p style="font-size: 16px; font-weight: bold;  color: #D3D3D3;padding: 0px; ">可以点击下方按钮唤醒<p/>
        ''')

        self.textLabel.setAlignment(Qt.AlignCenter)

        # 状态标签

        self.statusLabel = QLabel('当前状态：休眠中', self)
        self.statusLabel.setStyleSheet("""  
                    QLabel {  
                        font-size: 16px;  
                        font-weight: bold;  
                        color: #D3D3D3;    
                        padding: 0px; 
                    }  
                """)
        self.statusLabel.setAlignment(Qt.AlignCenter)


        # 按钮
        self.startButton = QPushButton('', self)
        self.startButton.setIconSize(QSize(300, 150))
        self.startButton.setStyleSheet("QPushButton { border: none; }")
        # AILight
        self.image_folder = './data/AILight'
        self.image_files = sorted([os.path.join(self.image_folder, f) for f in os.listdir(self.image_folder) if
                            f.endswith(('.png', '.jpg', '.jpeg'))])
        self.current_image_index = 0

        # 定时器
        self.image_timer = QTimer(self)
        self.image_timer.timeout.connect(self.updateButton)
        self.image_timer.start(50)  # 每500毫秒更换一次图片

        self.verticalLayout.addStretch(1)
        self.verticalLayout.addWidget(self.imageLabel)
        self.verticalLayout.addWidget(self.textLabel)
        self.verticalLayout.addStretch(1)
        # self.verticalLayout.addWidget(self.opLabel)
        self.verticalLayout.addWidget(self.statusLabel)
        self.verticalLayout.addStretch(1)
        self.verticalLayout.addWidget(self.startButton)
        self.verticalLayout.addStretch(4)


        self.startButton.clicked.connect(self.startConversation)
        self.state = '休眠中'



    def capture_voice(self):
        self.collect_voice_signal.emit()
    def stop_speech(self):
        self.stop_speech_signal.emit()
    def updateButton(self):
        # 更新按钮的图标
        if self.current_image_index < len(self.image_files):
            icon = QIcon(self.image_files[self.current_image_index])
            self.startButton.setIcon(icon)
            self.current_image_index = (self.current_image_index + 1) % len(self.image_files)

    def updateImage(self):
        if self.timecounter < 15:
            self.timecounter= self.timecounter+1
        else:
            self.timecounter=0
        if self.current_image_index1 < len(self.image_files1) and self.timecounter<4:
            pixmap = QPixmap(self.image_files1[self.current_image_index1])

            self.imageLabel.setPixmap(pixmap)
            self.current_image_index1 = (self.current_image_index1 + 1) % len(self.image_files1)

    def startConversation(self):
        if self.test_api_key():
            if self.state == '休眠中':
                self.updateStatus('聆听中')
                QApplication.processEvents()
                self.capture_voice()     #获取声音信息
                # self.respondToUser()
                #self.timer.start(5000)  # 3秒
            if self.state == '应答中':
                self.updateStatus('休眠中')
                self.stop_speech()



    def updateStatus(self, new_status):
        self.state = new_status
        print(self.state)
        self.statusLabel.setText(f'当前状态：{self.state}')
        self.statusLabel.repaint()
        
    def switchViewType(self):
        if PublicTypes.viewType == "sidebar":
            self.statusLabel.setStyleSheet("""  
                        QLabel {  
                            font-size: 16px;  
                            font-weight: bold;  
                            color: #D3D3D3;    
                            padding: 0px; 
                        }  
            """)
            self.textLabel.setText('''
            <p style="font-size: 16px; font-weight: bold;  color: #D3D3D3;padding: 0px; ">欢迎使用方德AI语音助手<p/>
            <p style="font-size: 16px; font-weight: bold;  color: #D3D3D3;padding: 0px; ">可以点击下方按钮唤醒<p/>
            ''')
            h = self.height - 380
            self.setFixedHeight(h)
            self.setFixedWidth(480  - 36)
        elif PublicTypes.viewType == "windows":
            self.statusLabel.setStyleSheet("""  
                        QLabel {  
                            font-size: 16px;  
                            font-weight: bold;  
                            color: black;    
                            padding: 0px; 
                        }  
            """)
            self.textLabel.setText('''
            <p style="font-size: 16px; font-weight: bold;  color: black;padding: 0px; ">欢迎使用方德AI语音助手<p/>
            <p style="font-size: 16px; font-weight: bold;  color: black;padding: 0px; ">可以点击下方按钮唤醒<p/>
            ''')
            self.setFixedHeight(560)
            self.setFixedWidth(1164)
        else:
            raise NotImplementedError("illegal type")

    @staticmethod
    def test_api_key():
        try:
            messages = [
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": "hello world"}
            ]
            response = dashscope.Generation.call(model='qwen-max', messages=messages)
            print(response)

            if response.status_code == 200:
                return True
            else:
                print(f"API key 测试失败，错误信息：{response.message}")
                return False
        except KeyError as e:
            print(f"响应中缺少必要的字段：{e}")
            return False
        except Exception as e:
            print(f"发生未知错误：{e}")
            return False

#  测试布局和交互效果
# from PyQt5.QtWidgets import *
# from PyQt5.QtCore import *
# from PyQt5.QtGui import *
#
# if __name__ == '__main__':
#     app = QApplication([])
#     widget = exChange()
#     widget.show()
#     app.exec_()


