#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import random
import sys

import dashscope
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
import PyQt5.QtGui
from pydantic.v1.config import get_config

from RoundedMenu import RoundedMenu
import os

from bubble_message import BubbleMessage, ChatWidget,MessageType,Notice, MeetingWidget
from bottomWgt import BottomUi
from ChatTask import ChatTask
from StreamChatTask import StreamChatTask
from LocalTask import LocalTask
from SpeechTask import SPeechTask
from VoiceRecognition import VoiceRecognition, VoiceCallback
from Speech import Speech
from Config import Config
from LocalAppTask import LocalAppTask
from AiDrawingTask import AiDrawingTask
from preGuidWidget import PreWidget, ModelType
from ServerCheck import ServerCheck
from PublicTypes import PublicTypes
from ChatTalk import ChatTalk
from MeetingBottomWgt import MeetingBottomWidget
from MeetingTask import  MeetingTask
from MeetingFile import MeetingFile
from SettingPage import SettingsPage
from AboutPage import AboutPage
from PyQt5 import QtWidgets
from exchangePage import exChange
from ApikeyConfigPage import ApiKeyConfigPage
from GetInstallStatus import GetInstallStatus

app_list = Config.get_app_list()
public_types = PublicTypes()

class ShadowWindow(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog) #Qt.Dialog主要用于在任务栏隐藏图标
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setObjectName('shadowindow')
        self.setStyleSheet(
            '''
            #shadowindow{
                border-top-left-radius: 15px;
                border-top-right-radius: 15px;
                border-bottom-left-radius: 15px;
                border-bottom-right-radius: 15px;
            }
            '''
        )
        self.resize(public_types.mainwin_width + 10, public_types.mainwin_height + 12)
        self.setMaximumWidth(public_types.mainwin_width)
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0,0,0,0)
        self.widget = Mainwin()
        self.widget.chatWgt.update()
        self.widget.chatWgt.verticalScrollBar().setValue(200)
        self.setWindowTitle("NFS AI")   # 设置窗口名
        self.setWindowIcon(QIcon('./data/icon.png'))  # 设置窗口图标
        
        self.right_bottom_index = self.get_right_bottom_index()

        self.stack = QStackedWidget(self)

        # 中间的标签
        spacer_left = QSpacerItem(100, 50, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.middle_label = QLabel()
        self.middle_label.setStyleSheet('color: white;font-size:16px;')
        self.middle_label.setAlignment(Qt.AlignCenter)
        self.middle_label.setText("智能问答")
        self.middle_label.hide()  # 初始状态隐藏，当后面点击窗口的时候进行显示

        #title
        self.title = MianWinTitle(self)

        self.aboutPage = AboutPage(self)
        self.settingPage = SettingsPage(self)
        # self.title.mousePressEvent = lambda event:

        # home button
        # self.home_btn = QPushButton(self.title)
        # self.home_btn.setFixedSize(24, 24)
        # self.home_btn.setFlat(True)
        # self.home_btn.setStyleSheet(
        #     '''
        #     QPushButton{background-color: transparent;border:none;color:white;}
        #     '''
        # )
        #
        # self.home_btn.setIcon(QIcon('./data/side_main.png'))
        # self.home_btn.setIconSize(QSize(24, 24));
        # self.home_btn.clicked.connect(
        #     lambda: self.widget.shift_model(False)
        # )
        # self.home_btn.setFixedSize(24, 24)
        # self.title.title_layout.addWidget(self.home_btn)



        #将弹性空间添加到布局中，确保其居中
        self.title.title_layout.addItem(spacer_left)
        self.title.title_layout.insertWidget(3, self.middle_label)


        #切换按钮
        self.exchange_btn = QPushButton()
        self.exchange_btn.setFixedSize(24, 24)
        self.exchange_btn.setFlat(True)
        self.exchange_btn.setStyleSheet(
            '''
            QPushButton{background-color: transparent;border:none;color:white;}
            '''
        )
        self.exchange_btn.setIcon(QIcon('./data/icon_side_exchange.png'))
        self.exchange_btn.setIconSize(QSize(24, 24))
        self.exchange_btn.clicked.connect(self.navigate_to_exchange)

        self.title.title_layout.addWidget(self.exchange_btn)

        #模式切换按钮
        self.mode_menu = RoundedMenu(self)
        self.ifly_action = QAction("讯飞星火", self.mode_menu, checkable=True)
        self.ali_action = QAction("阿里通义", self.mode_menu, checkable=True)
        # 添加菜单
        self.mode_menu.addAction(self.ifly_action)
        self.mode_menu.addAction(self.ali_action)
        # 添加互斥
        self.action_group = QActionGroup(self)
        self.action_group.setExclusive(True)
        self.action_group.addAction(self.ifly_action)
        self.action_group.addAction(self.ali_action)
        self.action_group.triggered.connect(self.model_to_exchange)
        # set default model
        self.ifly_action.setChecked(True)

        self.model_exchange_btn = QPushButton()
        self.model_exchange_btn.setFixedSize(24, 24)
        self.model_exchange_btn.setFlat(True)
        self.model_exchange_btn.setStyleSheet(
            '''
            QPushButton{background-color: transparent;border:none;color:white;}
            '''
        )
        self.model_exchange_btn.setIcon(QIcon('./data/icon_model_exchange.png'))
        self.model_exchange_btn.setIconSize(QSize(24, 24))
        self.model_exchange_btn.clicked.connect(self.show_menu)

     #   self.title.title_layout.addWidget(self.model_exchange_btn)
        
        # setting_ button
        self.setting_btn = QPushButton(self.title)
        self.setting_btn.setIcon(QIcon('./data/side_more.png'))
        self.setting_btn.setIconSize(QSize(24, 24))
        self.setting_btn.setFixedSize(24, 24)
        self.title.title_layout.addWidget(self.setting_btn)
        self.setting_btn.setStyleSheet(
            '''
            QPushButton{background-color:transparent;border:none;color:white;}
            QPushButton::menu-indicator { image: none; }
            '''
        )
        
        
        menu = SettingMenu(self)
        
        # 添加二级菜单
        sub_menu = menu.addMenu("显示模式")
        self.action_window = QAction( "窗口模式", self)
        sub_menu.addAction(self.action_window)
        self.action_window.triggered.connect(self.switchToWindow)
        self.action_sidebar = QAction(QIcon('./data/menu_select.png'), "侧边栏模式", self)
        sub_menu.addAction(self.action_sidebar)
        self.action_sidebar.triggered.connect(self.switchToSidebar)
        menu.sub_menu = sub_menu
        menu.setSubMenu()
        
        menu.addAction("设置", self.openSettingPage)
        menu.addSeparator()
        menu.addAction("关于", self.openAboutPage)  
        self.setting_btn.setMenu(menu)

        # close  button
        self.close_btn = QPushButton(self.title)
        self.close_btn.setFixedSize(24, 24)
        self.close_btn.setStyleSheet(
            '''
            QPushButton{background-color:transparent;border:none;color:white;}
            '''
        )

        self.close_btn.setIcon(QIcon('./data/side_exit.png'))
        self.close_btn.setIconSize(QSize(24, 24))
        self.close_btn.clicked.connect(self.main_window_close)
        self.close_btn.setFixedSize(24, 24)
        self.title.title_layout.addWidget(self.close_btn)

        layout.addWidget(self.title)
        layout.addWidget(self.widget)
        self.initDrag()
        self.switchToSidebar()
        self.move_to_right_bottom()

    def main_window_close(self):
        self.window_visible = False
        self.hide()

    #切换功能跳转函数
    def model_to_exchange(self):
        if self.ifly_action.isChecked():
            qDebug("ifly_action")
            Config.set_model_name('XINGHUO')

        if self.ali_action.isChecked():
            qDebug("ali_action")
            Config.set_model_name('TONGYI')

    def show_menu(self):
        self.mode_menu.exec(self.model_exchange_btn.mapToGlobal(self.model_exchange_btn.rect().bottomLeft()))

    def navigate_to_exchange(self):
        if self.widget.exchange.isHidden():
            self.widget.set_chat_model()
            self.widget.contentStackWgt.setCurrentIndex(1)
            self.widget.bottomStackWgt.setCurrentIndex(0)
            self.widget.exchange.show()
            self.widget.chatWgt.hide()
            self.widget.bottom_ui.hide()
            self.widget.meetingWgt.hide()
            self.widget.meeting_bottom_ui.hide()
        else:
            self.widget.set_chat_model()
            self.widget.shift_model(True)
            
    def get_right_bottom_index(self):
        # 获取屏幕总数和屏幕几何数据
        desktop = QDesktopWidget()
        screen_count = desktop.screenCount()
        target_screen_index = 0
        if screen_count > 1:
            # 多屏情况下，寻找最右下角的屏幕
            right_most = 0
            bottom_most = 0
            for i in range(screen_count):
                screen_geo = desktop.screenGeometry(i)
                if screen_geo.right() > right_most:
                    right_most = screen_geo.right()
                    target_screen_index = i
                if screen_geo.bottom() > bottom_most:
                    bottom_most = screen_geo.bottom()
                    target_screen_index = i
        else:
            target_screen_index = 0
            
        return target_screen_index


    def move_to_right_bottom(self):
            # 获取屏幕总数和屏幕几何数据
            desktop = QDesktopWidget()
            screen_count = desktop.screenCount()
            if screen_count > 1:
                # 获取目标屏幕的几何信息
                target_screen_geometry = desktop.screenGeometry(self.right_bottom_index)
            else:
                # 单屏情况下使用默认屏幕
                target_screen_geometry = desktop.screenGeometry()

            # 计算窗口移动位置到屏幕右下角
            right = target_screen_geometry.right()
            bottom = target_screen_geometry.bottom()
            self.move(right - self.width(), bottom - self.height())
            
    def move_to_center(self):
            # 获取屏幕总数和屏幕几何数据
            desktop = QDesktopWidget()
            screen_count = desktop.screenCount()
            if screen_count > 1:
                # 多屏情况下，用第一个屏幕
                target_screen_index = 0 
                # 获取目标屏幕的几何信息
                target_screen_geometry = desktop.screenGeometry(target_screen_index)
            else:
                # 单屏情况下使用默认屏幕
                target_screen_geometry = desktop.screenGeometry()

            # 计算窗口移动位置到屏幕右下角
            right = target_screen_geometry.right()
            bottom = target_screen_geometry.bottom()
            self.move(int(right/2 - self.width()/2), int(bottom/2 - self.height()/2))
        
    # 切换到窗口模式
    def switchToWindow(self):
        if PublicTypes.viewType != "windows":
            PublicTypes.viewType = "windows"
            icon = QIcon()
            icon.addPixmap(QPixmap("./data/microphone_input.png"), QIcon.Normal, QIcon.Off)
            self.widget.bottom_ui.bottomContent.ui.audioInputBtn.setIcon(icon)
            self.middle_label.setStyleSheet("color: black;  font-size:16px;")
            icon1 = QIcon()
            icon1.addPixmap(QPixmap("./data/send.png"), QIcon.Normal, QIcon.Off)
            self.widget.bottom_ui.bottomContent.ui.sendBtn.setIcon(icon1)
            
            self.action_window.setIcon(QIcon('./data/menu_select.png'))
            self.action_sidebar.setIcon(QIcon(""))
            self.setMaximumWidth(1200)
            # self.home_btn.setIcon(QIcon('./data/icon_home.png'))
            self.close_btn.setIcon(QIcon('./data/close.png'))
            self.setting_btn.setIcon(QIcon('./data/icon_setting.png'))
            self.exchange_btn.setIcon(QIcon('./data/icon_window_exchange.png'))
            self.model_exchange_btn.setIcon(QIcon('./data/icon_model_exchange_window.png'))
            self.widget.resize(public_types.mainwin_windows_width, public_types.mainwin_windows_height-260)
            self.widget.setMaximumHeight(public_types.mainwin_windows_height)
            self.resize(public_types.mainwin_windows_width, public_types.mainwin_windows_height)
            self.setMaximumHeight(public_types.mainwin_windows_height)
            self.move_to_center()
            self.widget.guidWgt.switchViewType()
            self.widget.chatWgt.switchViewType()
            self.widget.exchange.switchViewType()
            self.widget.meetingWgt.switchViewType()
            self.widget.meeting_bottom_ui.switchViewType()
            self.widget.bottom_ui.bottomContent.setQPlainTextEditStyle()
            self.title.switchViewType()

            self.widget.bottom_ui.bottomContent.setFixedHeight(120)
            self.widget.meeting_bottom_ui.setFixedHeight(120)
            self.widget.bottomStackWgt.setFixedHeight(160)

            self.show()
            self.raise_()
            self.activateWindow()
            
        else:
            pass
        
    # 切换到侧边栏模式
    def  switchToSidebar(self):
        if PublicTypes.viewType != "sidebar":
            PublicTypes.viewType = "sidebar"
            icon = QIcon()
            icon.addPixmap(QPixmap("./data/microphone_input_sidebar.png"), QIcon.Normal, QIcon.Off)
            self.widget.bottom_ui.bottomContent.ui.audioInputBtn.setIcon(icon)
            
            icon1 = QIcon()
            icon1.addPixmap(QPixmap("./data/send_sidebar.png"), QIcon.Normal, QIcon.Off)
            self.widget.bottom_ui.bottomContent.ui.sendBtn.setIcon(icon1)
            self.middle_label.setStyleSheet('color: white; font-weight: bold; font-size:16px;')
            self.action_window.setIcon(QIcon(""))
            self.action_sidebar.setIcon(QIcon('./data/menu_select.png'))
            # self.home_btn.setIcon(QIcon('./data/side_main.png'))
            self.close_btn.setIcon(QIcon('./data/side_exit.png'))
            self.setting_btn.setIcon(QIcon('./data/side_more.png'))
            self.exchange_btn.setIcon(QIcon('./data/icon_side_exchange.png'))
            self.model_exchange_btn.setIcon(QIcon('./data/icon_model_exchange.png'))
            self.widget.guidWgt.switchViewType()
            self.widget.chatWgt.switchViewType()
            self.widget.exchange.switchViewType()
            self.widget.meetingWgt.switchViewType()
            self.widget.meeting_bottom_ui.switchViewType()
            self.widget.setMaximumHeight(public_types.mainwin_height)
            self.widget.resize(public_types.mainwin_width, public_types.mainwin_height)
            
            
            self.widget.bottomStackWgt.setFixedHeight(180)
            # self.widget.bottom_ui.bottomContent.setFixedHeight(180)
            self.widget.bottom_ui.bottomContent.setQPlainTextEditStyle()
            self.widget.meeting_bottom_ui.setFixedHeight(180)
            
            self.setMaximumHeight(public_types.mainwin_height)
            self.resize(public_types.mainwin_width, public_types.mainwin_height)
            self.title.switchViewType()
            self.setMaximumWidth(public_types.mainwin_width)
            
            self.move_to_right_bottom()

            self.show()
            self.raise_()
            self.activateWindow()
            
        else:
            pass
        

    def paintEvent(self, event):
        # 创建一个QPainter对象，并为当前窗口提供绘图功能
        if PublicTypes.viewType == "sidebar":
            #侧边栏模式背景
            painter = QPainter(self)
            pixmap = QPixmap('./data/cblbj.png').scaled(self.size())
            painter.drawPixmap(0, 0, pixmap)
        elif PublicTypes.viewType == "windows":
            # 窗口模式背景
            # 画圆角
            painter = QPainter(self)
            painter.setRenderHint(QPainter.Antialiasing)  # 设置抗锯齿，让圆角更加平滑
            self.setAttribute(Qt.WA_TranslucentBackground, False)
            painter.setPen(QColor(0, 0, 0, 10))  # 黑色，透明度设置为10
            painter.setBrush(self.palette().window().color())  # 设置画刷为窗口背景颜色
            radius = 15  # 设置圆角半径

            bounds = self.rect()  # 获取窗口的边界
            # 绘制阴影
            shadow = QLinearGradient(bounds.topLeft(), bounds.bottomLeft())
            shadow.setColorAt(0, QColor(100, 100, 100, 50))

            # 设置阴影的模糊半径
            painter.setPen(QPen(shadow, 1))
            painter.drawRoundedRect(self.rect(), radius, radius)
        else:
            raise NotImplementedError("illegal type")   


    def initDrag(self):
        # 设置鼠标跟踪判断默认值
        self._move_drag = False

    def keyPressEvent(self, a0: PyQt5.QtGui.QKeyEvent) -> None:
        if a0.key() == Qt.Key_Escape:
            print("esc")
            self.widget.shift_model(False)
            # self.widget.speechTask.stop_speech()

    def mousePressEvent(self, event):
        # 重写鼠标点击的事件
        if (event.button() == Qt.LeftButton) :
            # 鼠标左键点击标题栏区域
            self._move_drag = True
            self.move_DragPosition = event.globalPos() - self.pos()
            event.accept()
 
    def mouseMoveEvent(self, QMouseEvent):
        # 判断鼠标位置切换鼠标手势
        self.setCursor(Qt.ArrowCursor)
        if Qt.LeftButton and self._move_drag:
            # 标题栏拖放窗口位置
            self.move(QMouseEvent.globalPos() - self.move_DragPosition)
            QMouseEvent.accept()
 
    def mouseReleaseEvent(self, QMouseEvent):
        # 鼠标释放后，各扳机复位
        self._move_drag = False

    def openApiKeyConfigPage(self):
        self.apiKeyConfigPage = ApiKeyConfigPage()
        self.apiKeyConfigPage.show()

    def openSettingPage(self):
        self.settingPage.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog | Qt.WindowStaysOnTopHint)
        #创建并显示设置页面
        self.settingPage.show()
        # 绑定设置页面的关闭信号到槽函数
        self.settingPage.closed.connect(self.onSettingPageClosed)
        # 设置窗口置顶
        self.settingPage.raise_()
        # #修改一直置顶
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog)
        # 移到画面中间
        self.settingPage.move_to_center()
        self.show()
        
    def openAboutPage(self):
        self.aboutPage.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog | Qt.WindowStaysOnTopHint)
        #创建并显示设置页面
        self.aboutPage.show()
        # 绑定设置页面的关闭信号到槽函数
        self.aboutPage.closed.connect(self.onSettingPageClosed)
        # 设置窗口置顶
        self.aboutPage.raise_()
        # #修改一直置顶
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Dialog)
        # 移到画面中间
        self.aboutPage.move_to_center()
        self.show()

    def onSettingPageClosed(self):
        # 重新置顶主窗口
        self.raise_()
        self.show()

    def show_middle_label(self, text):
        self.middle_label.setText(text.upper())
        self.middle_label.show()

    def mouse_press_event(self):
        self.widget.shift_model(False)
        self.middle_label.hide()

class Mainwin(QWidget):
    def __init__(self):
        super().__init__()
        self.setObjectName('mainwindow')
        self.setStyleSheet(
            '''
            #mainwindow{
                border-top-left-radius: 15px;
                border-top-right-radius: 15px;
                border-bottom-left-radius: 15px;
                border-bottom-right-radius: 15px;
            }
            '''
        )

        # 背景透明
        self.setAttribute(Qt.WA_TranslucentBackground, True)

        layout = QVBoxLayout()
        layout.setContentsMargins(18,0,18,12)
        self.resize(public_types.mainwin_width, public_types.mainwin_height)
        self.chatWgt = ChatWidget()
        self.chatWgt.hide()
        self.guidWgt = PreWidget()
        #新建exchage对象
        self.exchange = exChange()
        self.exchange.hide()
        self.meetingWgt = MeetingWidget()
        self.meetingWgt.hide()
        TEXT = MessageType.Text
        IMAGE = MessageType.Image
        GUID = MessageType.Guid
        self.chatGuideFlag = True
        self.meetingGuideFlag = True

        # ## 调用气泡信息
        # self.guid_title = BubbleMessage('我是您的智能助手小苏', '',GUID, 16, is_send=False)
        # self.chatWgt.add_message_item(self.guid_title)

        # self.guid_content = BubbleMessage('作为您的智能助手，我将为您提供个性化的操作系统体验,帮助您完成各种任务、回答问题、提供实用信息，以及提供智能建议和提示。', '',GUID, 12, is_send=False)
        # self.chatWgt.add_message_item(self.guid_content)

        self.serverCheck = ServerCheck()
        self.serverCheck.internet_change_signal.connect(self.internet_change_handle)
        self.serverCheck.start()

        self.chatWgt.add_vertical_spacer()

        self.waitingMessage = BubbleMessage('', '', MessageType.Waiting, 12, is_send=False)
        self.waitingMeetingMessage = BubbleMessage('', '', MessageType.Waiting, 12, is_send=False)

        self.sendTask = ChatTask()
        self.sendTask.complete_signal.connect(self.ai_callback)

        self.sendStreamTask = StreamChatTask()
        self.sendStreamTask.complete_signal.connect(self.ai_callback)
        self.sendStreamTask.message_ready_signal.connect(self.ai_stream_callback)
        self.temp_bubble_message = None

        self.localTask = LocalTask()
        self.localTask.complete_signal.connect(self.ai_callback)

        self.speechTask = SPeechTask()
        self.speechTask.set_callback(self.speechTaskCallback)
        self.localAppTask = LocalAppTask()
        self.voiceInput = VoiceRecognition()
        # self.voiceInput.VoiceRecognitionSignal.connect(self.send_voice_message_to_ai)
        self.voice_message = ""
        self.voiceInput.VoiceRecognitionSignal.connect(self.receive_voice_message)

        self.aiDrawingTask = AiDrawingTask()
        self.aiDrawingTask.complete_signal.connect(self.complete_drawing_handle)

        self.bottom_ui = BottomUi()
        self.bottom_ui.setMaximumHeight(240)
        self.bottom_ui.hide()

        self.meetingTask = MeetingTask()
        self.meetingTask.message_ready_signal.connect(self.meeting_message_ready_handle)
        self.meetingTask.meeting_status_signal.connect(self.meeting_service_status_handle)

        self.meeting_bottom_ui = MeetingBottomWidget()
        self.meeting_bottom_ui.hide()
        self.meeting_bottom_ui.switch_signal.connect(self.meeting_signal_handle)

        # self.bottom_ui.setMaximumWidth(420)
        self.bottom_ui.bottomContent.collect_voice_signal.connect(self.collect_voice_handle)
        self.bottom_ui.bottomContent.send_text_signal.connect(self.send_message_to_ai)
        # self.bottom_ui.send_text_signal.connect(self.send_message_to_ai)

        self.exchange.collect_voice_signal.connect(self.collect_voice_handle)
        self.exchange.stop_speech_signal.connect(self.stop_speech_signal_exchange)

        self.guidWgt.chat_model_signal.connect(self.set_chat_model)
        self.guidWgt.paint_model_signal.connect(self.set_paint_model)
        self.guidWgt.meeting_minuts_model_signal.connect(self.set_meeting_minuts_model)

        self.bottom_ui.bottomContent.ui.audioInputBtn.setDisabled(True)

        bottom_spacer = QSpacerItem(10, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.contentStackWgt = QStackedWidget()
        self.contentStackWgt.addWidget(self.guidWgt)
        self.contentStackWgt.addWidget(self.exchange)
        self.contentStackWgt.addWidget(self.chatWgt)
        self.contentStackWgt.addWidget(self.meetingWgt)
        self.contentStackWgt.setCurrentIndex(0)

        layout.addWidget(self.contentStackWgt)
        # layout.addWidget(self.meetingWgt)
        # layout.addWidget(self.guidWgt)
        layout.addItem(bottom_spacer)
        self.bottomStackWgt = QStackedWidget()
        self.bottomStackWgt.addWidget(self.bottom_ui)
        self.bottomStackWgt.addWidget(self.meeting_bottom_ui)
        self.bottomStackWgt.setCurrentIndex(0)
        self.bottom_ui.hide()
        layout.addWidget(self.bottomStackWgt)
        # layout.addWidget(self.meeting_bottom_ui)
        self.setLayout(layout)

        self.bottom_ui.setEnabled(False)

        self.timer = QTimer(self)  # 定义定时器
        self.timer.timeout.connect(self.start_meeting)  # 定时器信号连接到updateImage方法
        # # 聆听中到等待中的自动转换
        self.last_voice_message = ""
        self.timerSend = QTimer(self)
        self.timerSend.timeout.connect(self.listeningToWaiting)

        # # 聆听中到等待中的自动转换函数
    def listeningToWaiting(self):
        if self.voice_message != self.last_voice_message:
            self.last_voice_message = self.voice_message
        else:
            self.exchange.capture_voice()
            # self.exchange.updateStatus('等待中')

    def show_meeting_waiting_message(self, show):    #总是和BubbleMessage成对出现，send为True则显示，否则隐藏，待优化
        if show:
            self.meetingWgt.add_message_item(self.waitingMeetingMessage)
            self.waitingMeetingMessage.show()
        else:
            self.meetingWgt.remove_message_item(self.waitingMeetingMessage)
            self.waitingMeetingMessage.hide()

    def meeting_service_status_handle(self, status):
        print(status)
        message = status
        self.meetingTask.status = "StartFail"
        self.show_meeting_waiting_message(False)
        if "Timeout" in status and "https://tingwu.cn-beijing.aliyuncs.com" in status:
            message = "连接服务器超时，请检查网络环境..."

        if "Failed to connect to host or proxy" in status:
            message = "连接服务器失败，请检查网络环境..."

        if "create transcriber request fail." in status:
            message = "创建会议记录请求失败..."

        if "transcriber request start fail." in status:
            message = "会议记录请求启动失败..."

        if "transcriber request start fail." in status:
            message = "会议记录请求启动超时..."

        if "open default PCM device fail." in status:
            message = "打开语音输入设备失败..."

        if "send audio fail" in status:
            message = "发送音频流失败，请检查网络环境..."

        bubble_message = BubbleMessage(message, '', Type=MessageType.Text, font_size=12, is_send=False)
        bubble_message.message.meeting_content_adjust_size()
        self.meetingWgt.add_message_item(bubble_message)
        bubble_message.playBtn.hide()
        self.meeting_bottom_ui.meeting_switch_handel()

    def meeting_message_ready_handle(self, message):
        self.show_meeting_waiting_message(False)
        if "StartFail" in self.meetingTask.status:
            self.meetingTask.status = ""
            return
        else:
            bubble_message = BubbleMessage(message, '', Type=MessageType.Text, font_size=12, is_send=False)
            # bubble_message.switch_signal.connect(self.switch_signal_handle)
            bubble_message.message.meeting_content_adjust_size()
            self.meetingWgt.add_message_item(bubble_message)
            bubble_message.playBtn.hide()
            self.show_meeting_waiting_message(True)
            if "您已结束会议，会议纪要文件" not in message:
                MeetingFile.write_temp_file(message)

    def start_meeting(self):
        self.meetingTask.start()
        self.meetingTask.start_task()
        self.timer.stop()

    def meeting_signal_handle(self, status):
        if status:
            self.meetingTask.close_bin_process()
            self.meetingTask.exec_bin_process()
            self.timer.start(500)
            self.show_meeting_waiting_message(True)
        else:
            self.meetingTask.stop_task()
            self.show_meeting_waiting_message(False)

    def internet_change_handle(self, status):
        if status:
            self.bottom_ui.bottomContent.ui.audioInputBtn.setEnabled(True)
        else:
            self.bottom_ui.bottomContent.ui.audioInputBtn.setDisabled(True)

    def switch_signal_handle(self, flag, message, bubble_message):
        if not flag:
            self.speechTask.stop_speech()
        else:
            self.speechTask.set_message(message, bubble_message)
            self.speechTask.start()
    def stop_speech_signal_exchange(self):
        self.speechTask.stop_speech()

    def speechTaskCallback(self, bubble_message):
        print("speechTaskCallback")
        bubble_message.playComplete()
        self.exchange.updateStatus('休眠中')

    def showExample(self):
        print('show example')
        if self.chatGuideFlag:
            if self.modeType == ModelType.Chat or self.modeType == ModelType.Paint:
                self.chatGuideFlag = False
                self.chatWgt.clearLayout()
                message = '您可以这样对我说： \n“打开应用商店”、\n“调高音量”、\n“今天北京的天气如何”、\n“周一上午9点有会议”'
                if PublicTypes.viewType=="sidebar":
                    bubble_message = BubbleMessage(message, '', MessageType.Text, 12, is_send=False) #生成气泡label
                    bubble_message.message.setStyleSheet(
                            '''
                            background-color: rgba(54,54,54,204);
                            border-radius:10px;
                            padding:10px;
                            color: white;
                            '''
                        )
                if PublicTypes.viewType=="windows":
                    bubble_message = BubbleMessage(message, '', MessageType.Text, 12, is_send=False) #生成气泡label
                    bubble_message.message.setStyleSheet(
                            '''
                            background-color: rgba(204,204,204,204);
                            border-radius:10px;
                            padding:10px;
                            color: white;
                            '''
                        )
                    
                print("PublicTypes.viewType", PublicTypes.viewType)
                bubble_message.switch_signal.connect(self.switch_signal_handle)
                self.chatWgt.add_message_item(bubble_message)  #添加到界面
                if self.serverCheck.internet_status():
                    if Config.get_text_play_flag() == "True":
                        self.speechTask.set_message(message, bubble_message)
                        self.speechTask.start()

                bubble_message.playBtn.hide()
        if self.meetingGuideFlag:
            if self.modeType == ModelType.MeetingMinuts:
                self.meetingGuideFlag = False
                bubble_message = BubbleMessage('会议记录 \n方德AI助手将会全程记录会议内容，并为您总结一份会议纪要。现在您可以点击下方按钮开始会议', '',
                                               MessageType.Text, 12, is_send=False)  # 生成气泡label
                bubble_message.message.meeting_content_adjust_size()
                self.meetingWgt.add_message_item(bubble_message)  # 添加到界面
                bubble_message.playBtn.hide()

    def shift_model(self, toModel):
        if toModel == True:
            if self.modeType == ModelType.MeetingMinuts:
                self.contentStackWgt.setCurrentIndex(3)
                self.bottomStackWgt.setCurrentIndex(1)
                self.meetingWgt.show()
                self.showExample()
                self.meeting_bottom_ui.show()
                self.exchange.hide()
                self.bottom_ui.hide()
                self.chatWgt.hide()

            else:
                self.contentStackWgt.setCurrentIndex(2)
                self.bottomStackWgt.setCurrentIndex(0)
                self.chatWgt.show()
                self.exchange.hide()
                self.showExample()
                self.bottom_ui.setEnabled(True)
                self.bottom_ui.show()
                self.meetingWgt.hide()
                self.meeting_bottom_ui.hide()
            self.guidWgt.hide()
        else:
            self.contentStackWgt.setCurrentIndex(0)
            self.bottomStackWgt.setCurrentIndex(0)
            self.guidWgt.show()
            self.chatWgt.hide()

            self.exchange.hide()
            # self.bottom_ui.setEnabled(False)
            self.bottom_ui.hide()
            self.meetingWgt.hide()
            self.meeting_bottom_ui.hide()

    def set_chat_model (self):
        print("set_chat_model")
        if Config.get_tongyi_api_key() == "":
            self.parent().openSettingPage()
        self.modeType = ModelType.Chat
        self.parent().show_middle_label("智能问答")
        self.shift_model(True)
        self.bottom_ui.updateMenu(ModelType.Chat)

    def set_paint_model (self):
        print("set_paint_model")
        if Config.get_tongyi_api_key() == "":
            self.parent().openSettingPage()
        self.modeType = ModelType.Paint
        self.parent().show_middle_label("绘画制作")
        self.shift_model(True)
        self.bottom_ui.updateMenu(ModelType.Paint)

    def set_meeting_minuts_model (self):
        print("set_meeting_minuts_model")
        if Config.get_tongyi_access_key_id() == "" or Config.get_tongyi_app_key() == "" or Config.get_tongyi_access_key_secret() == "" or  (not GetInstallStatus.is_package_installed("tingwu-meettings-plugin")):
            self.parent().openSettingPage()
        self.modeType = ModelType.MeetingMinuts
        self.parent().show_middle_label("会议记录")
        self.shift_model(True)
        
    def set_AiAss_model(self):
        print("set_AiAss_model")
        self.modeType = ModelType.AiAss
        self.shift_model(True)


    def show_waiting_message(self,show):    #总是和BubbleMessage成对出现，send为True则显示，否则隐藏，待优化
        if show:
            self.chatWgt.add_message_item(self.waitingMessage)
            self.waitingMessage.show()
        else:
            self.chatWgt.remove_message_item(self.waitingMessage)
            self.waitingMessage.hide()

    def complete_drawing_handle(self, file_path):
        self.show_waiting_message(False)
        if "fail:" in file_path:
            bubble_message = BubbleMessage(file_path, '', Type=MessageType.Text, font_size=12, is_send=False)
            self.chatWgt.add_message_item(bubble_message)
            bubble_message.playBtn.hide()
        else:
            bubble_message = BubbleMessage(file_path, '', Type=MessageType.Image, font_size=12, is_send=False)
            self.chatWgt.add_message_item(bubble_message)
            Speech.short_text_play("绘图完成")

        self.bottom_ui.bottomContent.set_send_button_status(True)

    def collect_voice_handle(self):
        print("collect_voice_handle")
        if self.test_api_key():
            # print("API_KEY测试成功！")
            if self.bottom_ui.bottomContent.collect_voice_flag:
                self.bottom_ui.bottomContent.collect_voice_flag = False
                self.voiceInput.stop_recognition()
                self.bottom_ui.bottomContent.stop_timer()
                self.send_voice_message_to_ai(self.voice_message)
                self.timerSend.stop()
                self.voice_message = ""
                self.bottom_ui.bottomContent.ui.inputLineEdit.setPlainText("")
            else:
                self.bottom_ui.bottomContent.ui.inputLineEdit.setPlainText("")
                self.voice_message = ""
                self.bottom_ui.bottomContent.collect_voice_flag = True
                self.bottom_ui.bottomContent.start_timer()
                self.voiceInput.start()
                self.timerSend.start(3000)
        # else:
        #     print("API_KEY测试失败！")

    @staticmethod
    def test_api_key():
        try:
            messages = [
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": "hello world"}
            ]
            response = dashscope.Generation.call(model='qwen-max', messages=messages)
            print(response)

            if response.status_code == 200:
                return True
            else:
                print(f"API key 测试失败，错误信息：{response.message}")
                return False
        except KeyError as e:
            print(f"响应中缺少必要的字段：{e}")
            return False
        except Exception as e:
            print(f"发生未知错误：{e}")
            return False

    def start_local_app(self, text):
        flag = False
        for dic in app_list:
            for key in dic:
                print(key+":"+text)
                if key in text:
                    print("正在为您" + key)
                    if self.serverCheck.internet_status():
                        if Config.get_text_play_flag() == "True":
                            self.speechTask.set_message("正在为您" + key)
                            self.speechTask.start()
                    # os.system(dic[key])

                    self.localAppTask.set_app(dic[key])
                    self.localAppTask.start()
                    return
        if not flag:
            print("抱歉，暂不支持此操作")
            bubble_message = BubbleMessage("抱歉，暂不支持此操作", '', Type=MessageType.Text, font_size=12, is_send=False)
            self.chatWgt.add_message_item(bubble_message)
            if self.serverCheck.internet_status():
                if Config.get_text_play_flag() == "True":
                    self.speechTask.set_message("抱歉，暂不支持此操作")
                    self.speechTask.start()
            else:
                bubble_message.playBtn.hide()

    def receive_voice_message(self, message):
        self.voice_message += message
        self.bottom_ui.bottomContent.ui.inputLineEdit.setPlainText(self.voice_message)

    def send_paint_to_ai(self, message):
        if self.serverCheck.internet_status():
            # 文字绘图
            if Config.get_text_play_flag() == "True":
                self.speechTask.set_message("好的，请稍等")
                self.speechTask.start()
            self.aiDrawingTask.set_prompt(message)
            self.aiDrawingTask.start()
            self.aiDrawingTask.start_timer(30000)
        else:
            bubble_message = BubbleMessage("无法连接互联网服务，请检查网络", '', Type=MessageType.Text, font_size=12, is_send=False)
            self.chatWgt.add_message_item(bubble_message)
            bubble_message.playBtn.hide()
            self.show_waiting_message(False)
            self.bottom_ui.bottomContent.set_send_button_status(True)

    def send_quest_to_ai(self, message):
        self.exchange.updateStatus('等待中')

        if Config.get_chat_model_name() == "DEEPSEEKLOCAL":
            self.sendStreamTask.set_topic(message)
            self.sendStreamTask.start()
        elif Config.get_chat_model_name() == "DEEPSEEK":
            self.sendStreamTask.set_topic(message)
            self.sendStreamTask.start()
        else:

            if self.serverCheck.internet_status():
                if Config.get_text_play_flag() == "True":
                    Speech.short_text_play("好的，请稍等")
                self.sendTask.set_topic(message)
                self.sendTask.start()
            elif self.serverCheck.localserver_status():
                self.localTask.set_topic(message)
                self.localTask.start()
            else:
                bubble_message = BubbleMessage("服务异常", '', Type=MessageType.Text, font_size=12, is_send=False)
                self.chatWgt.add_message_item(bubble_message)
                bubble_message.playBtn.hide()
                self.show_waiting_message(False)
                self.bottom_ui.bottomContent.set_send_button_status(True)
                self.exchange.updateStatus('休眠中')

    def send_voice_message_to_ai(self, message):
        if len(message) == 0:
            self.exchange.updateStatus('休眠中')
            self.bottom_ui.bottomContent.collect_voice_flag = False
            return

        bubble_message = BubbleMessage(message, '', Type=MessageType.Text, font_size=12, is_send=True)
        self.chatWgt.add_message_item(bubble_message)
        self.show_waiting_message(True)

        if "返回" in message or "返回首页" in message:
            self.shift_model(False)
            self.show_waiting_message(False)
            return

        #打开本地应用
        if "打开" in message or "调高" in message or "调低" in message or "关闭" in message or "调节" in message \
                or "查询" in message:
            self.start_local_app(message)
            self.show_waiting_message(False)
            return

        if self.modeType == ModelType.Chat or self.modeType == ModelType.MeetingMinuts:
            # AI模型检索
            # Speech.short_text_play("好的，请稍等")
            # self.sendTask.set_topic(message)
            # self.sendTask.start()
            self.send_quest_to_ai(message)
        elif self.modeType == ModelType.Paint:
            #文字绘图
            # self.speechTask.set_message("好的，请稍等")
            # self.speechTask.start()
            # self.aiDrawingTask.set_prompt(message)
            # self.aiDrawingTask.start()
            self.send_paint_to_ai(message)
        else:
            print("MeetingMinuts")

    def calculate_string_width(self, text, font_size=12, font_family='微软雅黑'):
        # 设置字体
        font = QFont(font_family, font_size)

        # 使用 QFontMetrics 来计算字符串的宽度
        font_metrics = QFontMetrics(font)
        width = font_metrics.horizontalAdvance(text)

        print(text+":")
        print(width)

        return width

    def ai_stream_callback(self, result, flag):
        self.show_waiting_message(False)
        width = self.calculate_string_width(result, 12, '微软雅黑')
        if flag:
            bubble_message = BubbleMessage(result, '', Type=MessageType.Text, font_size=12, is_send=False)
            bubble_message.switch_signal.connect(self.switch_signal_handle)
            self.chatWgt.add_message_item(bubble_message)
            self.temp_bubble_message = bubble_message
        else:
            self.temp_bubble_message.message.setText(result)

        self.temp_bubble_message.message.adjust_size(result)

        if self.serverCheck.internet_status():
            if "Invalid API-key provided" in result:
                self.temp_bubble_message.playBtn.hide()
            else:
                if Config.get_text_play_flag() == "True":
                    self.speechTask.set_message(result, self.temp_bubble_message)
                    self.speechTask.start()
        else:
            self.temp_bubble_message.playBtn.hide()

        # if not self.voiceStatus:
        self.bottom_ui.bottomContent.set_send_button_status(True)

    def ai_callback(self, result):
        self.show_waiting_message(False)
        bubble_message = BubbleMessage(result, '', Type=MessageType.Text, font_size=12, is_send=False)
        self.exchange.updateStatus('应答中')
        bubble_message.switch_signal.connect(self.switch_signal_handle)
        self.chatWgt.add_message_item(bubble_message)

        if self.serverCheck.internet_status():
            if Config.get_text_play_flag() == "True":
                self.speechTask.set_message(result, bubble_message)
                self.speechTask.start()
            if "Invalid API-key provided" in result:
                bubble_message.playBtn.hide()
                self.exchange.updateStatus('休眠中')
        else:
            bubble_message.playBtn.hide()
            self.exchange.updateStatus('休眠中')

        # if not self.voiceStatus:
        self.bottom_ui.bottomContent.set_send_button_status(True)

    def send_message_to_ai(self, message):
        print("send_message_to_ai:" + message)
        bubble_message = BubbleMessage(message, '', Type=MessageType.Text, font_size=12, is_send=True)
        self.chatWgt.add_message_item(bubble_message)
        self.show_waiting_message(True)

        if "返回" in message or "返回首页" in message:
            self.shift_model(False)
            self.show_waiting_message(False)
            self.bottom_ui.bottomContent.set_send_button_status(True)
            return

        # 打开本地应用
        if "打开" in message or "调高" in message or "调低" in message or "关闭" in message or "调节" in message \
                or "查询" in message:
            self.bottom_ui.bottomContent.set_send_button_status(True)
            self.start_local_app(message)
            self.show_waiting_message(False)
            return

        if self.modeType == ModelType.Chat or self.modeType == ModelType.MeetingMinuts:
            # AI模型检索
            # Speech.short_text_play("好的，请稍等")
            # self.sendTask.set_topic(message)
            # self.sendTask.start()
            self.send_quest_to_ai(message)
        elif self.modeType == ModelType.Paint:
            #文字绘图
            # self.speechTask.set_message("好的，请稍等")
            # self.speechTask.start()
            # self.aiDrawingTask.set_prompt(message)
            # self.aiDrawingTask.start()
            # self.aiDrawingTask.start_timer(30000)
            self.send_paint_to_ai(message)
        else:
            print("MeetingMinuts")
            bubble_message = BubbleMessage("会议纪要暂未实现", '', Type=MessageType.Text, font_size=12, is_send=False)
            self.chatWgt.add_message_item(bubble_message)
            self.show_waiting_message(False)

    def currentIndexChanged(self, index):
        print('index = %d', index)

    # def shiftToTextInput(self):
    #     print('shiftToTextInput')
    #     self.bottom_ui.setCurrentIndex(1)
    #     self.bottom_ui.setFixedHeight(80)

    # def shiftToAudioInput(self):
    #     self.bottom_ui.setCurrentIndex(0)
    #     self.bottom_ui.setFixedHeight(160)

    def set_background_color(self, color):
        palette = QPalette()
        palette.setColor(QPalette.Background, color)
        self.setPalette(palette)
        self.setAutoFillBackground(True)

    def blur(self):
        blur = QGraphicsBlurEffect()
        blur.setBlurRadius(10)
        blur.setBlurHints(QGraphicsBlurEffect.QualityHint)
        self.setGraphicsEffect(blur)

    def value(self, val):
        print('pos:', val)
        print('滚动条最大值', self.chatWgt.verticalScrollBar().maximum())


#菜单类
class SettingMenu(QMenu):
    def __init__(self, *args, **kwargs):
        super(SettingMenu, self).__init__()
        self.radius = 10
        self.setting_menu_sytle =  '''
            QMenu {{
                /* 半透明效果 */
                border-radius: {radius};
                border: 2px solid rgb(255, 255, 255);
                background-color: rgba(255, 255, 255, 230);
            }}
            QMenu::item {{
                border-radius: {radius};
                /* 这个距离很麻烦需要根据菜单的长度和图标等因素微调 */
                padding: 8px 48px 8px 12px; /* 12px是文字距离左侧距离*/
                background-color: transparent;
            }}

            /* 鼠标悬停和按下效果 */
            QMenu::item:selected {{
                border-radius: {radius};
                /* 半透明效果 */
                background-color: rgba(230, 240, 255, 232);
            }}

            /* 禁用效果 */
            QMenu::item:disabled {{
                border-radius: {radius};
                background-color: transparent;
            }}

            /* 图标距离左侧距离 */
            QMenu::icon {{
                left: 15px;
            }}

            /* 分割线效果 */
            QMenu::separator {{
                height: 1px;
                background-color: rgb(232, 236, 243);
            }}'''.format(radius=self.radius)
        self.setStyleSheet(self.setting_menu_sytle)
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setWindowFlags(self.windowFlags()  | Qt.FramelessWindowHint )

    def setSubMenu(self):
        self.sub_menu.setStyleSheet(self.setting_menu_sytle)
        self.sub_menu.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setWindowFlags(self.windowFlags()  | Qt.FramelessWindowHint )
        
# 上方标题栏
class MianWinTitle(QWidget):
    def __init__(self, parent=None):
        super(MianWinTitle, self).__init__(parent)
        self.setAttribute(Qt.WA_StyledBackground)
        self.setObjectName('title')
        self.setStyleSheet(
            '''
            #title{
                border-top-left-radius: 5px;
                border-top-right-radius: 5px;
                background:transparent;
            }
            '''
        )
        self.installEventFilter(self)
        self.setFixedHeight(36)
        self.icon = QLabel(self)
        self.icon.mousePressEvent = lambda event: self.parent().mouse_press_event()
        self.icon.setFixedSize(16,16)
        pixmap = QPixmap('./data/icon.png')
        self.icon.setPixmap(pixmap)
        self.icon.setScaledContents(True)
        self.title_layout = QHBoxLayout(self)
        self.title_layout.setContentsMargins(18, 12, 18, 0)
        self.title_layout.addWidget(self.icon)

        self.title_name = QLabel()
        self.title_name.mousePressEvent = lambda event: self.parent().mouse_press_event()
        self.title_name.setText('方德AI助手')
        self.title_name.setStyleSheet(
            '''
            color:white;
            font-size:16px;
            '''
        )

        self.title_layout.addWidget(self.title_name)

        title_spacer = QSpacerItem(40, 10, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.title_layout.addItem(title_spacer)
        
    def switchViewType(self):
        if PublicTypes.viewType == "sidebar":
            self.title_layout.setContentsMargins(18, 12, 18, 0)
            self.setStyleSheet(
                '''
                #title{
                    border-top-left-radius: 5px;
                    border-top-right-radius: 5px;
                    background:transparent;
                }
                '''
            )
            self.icon.setFixedSize(16,16)
            pixmap = QPixmap('./data/icon.png')
            self.icon.setPixmap(pixmap)
            self.icon.setScaledContents(True)
            self.title_name.setStyleSheet(
                '''
                color:white;
                font-size:16px;
                font-weight:bold;
                '''
            )
            self.title_layout.setSpacing(10)
            
        elif PublicTypes.viewType == "windows":
            self.title_layout.setContentsMargins(18, 6, 18, 6)
            self.setStyleSheet(
                '''
                #title{
                    border-left: 2px solid transparent;
                    border-top-left-radius: 15px;
                    border-top-right-radius: 15px;
                    background-color: white;
                    background-clip: padding-box; 
                }
                '''
            )
            self.icon.setFixedSize(20, 20)
            pixmap = QPixmap('./data/icon_win.png')
            self.icon.setPixmap(pixmap)
            self.icon.setScaledContents(True)
            
            self.title_name.setStyleSheet(
                '''
                color:black;
                font-size: 16px;
                font-weight: 400;
                '''
            )
            self.title_layout.setSpacing(10)
            
            
        else:
            raise NotImplementedError("illegal type")

# if __name__ == '__main__':
#     app = QApplication([])
#     widget = ShadowWindow()
#     widget.show()
#     app.exec_()
