# -*- coding: utf-8 -*-
#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#


from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import  QWidget
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtGui import QPixmap, QIcon
from ChatPlainTextEdit import ChatPlainTextEdit
from PublicTypes import PublicTypes


class ModelSelectWidget(QWidget):
        collect_voice_signal = pyqtSignal()
        collect_voice_flag = False
        send_text_signal = pyqtSignal(str)

        def __init__(self):
                super().__init__()
                self.setObjectName('modelSelectWidget')
                self.setStyleSheet(
                       '''
                        #bottomWidget{
                                border: 0px;
                        }
                        '''
                )

                self.setWindowFlags(Qt.FramelessWindowHint)
                self.ui = Ui_modelSelectWidget()
                self.ui.setupUi(self)
                self.setContextMenuPolicy(Qt.NoContextMenu)

                self.current_image_index = 0
                self.total_images =23  # 图片总数-1


        def setPlainEditStyle(self):
                self.ui.inputLineEdit.setStyleSheet(
                '''
                QScrollBar:vertical {
                border-width: 0px;
                border: none;
                background:rgba(0, 0, 0, 0);
                width:5px;
                margin: 0px 0px 0px 0px;
                }
                QScrollBar::handle:vertical {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                stop: 0 #DDDDDD, stop: 0.5 #DDDDDD, stop:1 #aaaaff);
                min-height: 20px;
                max-height: 20px;
                margin: 0 0px 0 0px;
                border-radius: 2px;
                }
                QScrollBar::add-line:vertical {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                stop: 0 rgba(64, 65, 79, 0), stop: 0.5 rgba(64, 65, 79, 0),  stop:1 rgba(64, 65, 79, 0));
                height: 0px;
                border: none;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
                }
                QScrollBar::sub-line:vertical {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                stop: 0  rgba(64, 65, 79, 0), stop: 0.5 rgba(64, 65, 79, 0),  stop:1 rgba(64, 65, 79, 0));
                height: 0 px;
                border: none;
                subcontrol-position: top;
                subcontrol-origin: margin;
                }
                QScrollBar::sub-page:vertical {
                background: rgba(64, 65, 79, 0);
                }

                QScrollBar::add-page:vertical {
                background: rgba(64, 65, 79, 0);
                }

                QPlainTextEdit {
                        background:transparent;
                        color:#333333;
                        font: 14px "Sans Serif";
                }
                '''
                )

        def setQPlainTextEditStyle(self):
            # 根据窗口类型设置不同的文字颜色样式
            if PublicTypes.viewType == "sidebar":
                color_style = "color: white;"
            else:
                color_style = "color: #333333;"

            self.ui.inputLineEdit.setStyleSheet(
                f'''
                QPlainTextEdit {{
                    {color_style}
                    background: transparent;
                    font: 14px "Sans Serif";
                }}
                '''
            )

        def paintEvent(self, event):
                painter = QPainter(self)
                # print(self.size())
                painter.setOpacity(0.80)
                painter.setRenderHint(QPainter.Antialiasing)  # 设置抗锯齿，让圆角更加平滑
                # # 定义画笔和填充
                if PublicTypes.viewType =="sidebar":
                    painter.setBrush(QBrush(QColor(48, 66, 92)))  # 画刷颜色设置为白色
                    painter.setPen(QPen(QColor(48, 66, 92), 0))  # 画笔颜色和粗细
                else:
                    painter.setBrush(QBrush(QColor(189,195,209)))  # 画刷颜色设置为白色
                    painter.setPen(QPen(QColor(189,195,209), 0))  # 画笔颜色和粗细
                # # 绘制圆角矩形
                painter.drawRoundedRect(self.rect(), 10, 10)  # 设置边角为10像素的圆度



class Ui_modelSelectWidget(QWidget):
    def setupUi(self, modelSelectWidget):
        modelSelectWidget.setObjectName("modelSelectWidget")
        modelSelectWidget.resize(513, 180)
        modelSelectWidget.setMinimumSize(QtCore.QSize(384, 180))
        modelSelectWidget.setStyleSheet("#modelSelectWidget {\n"
        "    background-color: rgba(48,66,92, 204);\n"
        "}")
        self.verticalLayout = QtWidgets.QVBoxLayout(modelSelectWidget)
        # self.verticalLayout.setContentsMargins(24, 24, 26 ,12)
        self.verticalLayout.setObjectName("verticalLayout")
        self.inputLineEdit = ChatPlainTextEdit(modelSelectWidget)
        self.inputLineEdit.setStyleSheet("font: 14px \"Sans Serif\";\n"
"color:white;\n"
"background:transparent;")
        self.inputLineEdit.setObjectName("inputLineEdit")
        self.verticalLayout.addWidget(self.inputLineEdit)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, 12, -1)
        self.horizontalLayout.setSpacing(19)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(modelSelectWidget)
        QtCore.QMetaObject.connectSlotsByName(modelSelectWidget)

    def retranslateUi(self, modelSelectWidget):
        _translate = QtCore.QCoreApplication.translate
        modelSelectWidget.setWindowTitle(_translate("modelSelectWidget", "Form"))


#  测试布局和交互效果
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

if __name__ == '__main__':
    app = QApplication([])
    widget = ModelSelectWidget()
    widget.show()
    app.exec_()
