# -*- coding: utf-8 -*-
#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#


from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from bubble_message import BubbleMessage,MessageType
from PublicTypes import PublicTypes
import os
import time
public_types = PublicTypes()

class ModelType:
    Chat = 1
    Paint = 2
    MeetingMinuts = 3


class PreWidget(QWidget):
        chat_model_signal = pyqtSignal()
        paint_model_signal = pyqtSignal()
        meeting_minuts_model_signal = pyqtSignal()

        def __init__(self):
            super().__init__()
            self.setObjectName('pre')
            self.setStyleSheet(
                    '''
                    #pre{
                            border: 0px;
                            background: transparent;
                    }
                    '''
            )
            self.setMaximumHeight(922)
            # 获取显示器分辨率大小
            self.desktop = QApplication.desktop()
            self.screenRect = self.desktop.screenGeometry()
            self.height = self.screenRect.height()
            h = self.height - 380
            self.setFixedHeight(h)

            # self.setWindowFlags(Qt.FramelessWindowHint)
            
            # AIFace
            self.imageLabel = QLabel(self)
            self.imageLabel.setAlignment(Qt.AlignCenter)
            self.imageLabel.setStyleSheet("QLabel { padding: 0px;}")
            self.imageLabel.setAlignment(Qt.AlignCenter)
            
            # 图片列表和索引
            self.image_folder1 = './data/AIFace'

            self.image_files1 = [os.path.join(self.image_folder1, f) for f in os.listdir(self.image_folder1) if
                                f.endswith(('.png', '.jpg', '.jpeg'))]
            
            self.current_image_index1 = 0
            self.timecounter=0
            self.image_timer1 = QTimer(self)
            
            #wink
            self.image_timer1.timeout.connect(self.updateImage)
            self.image_timer1.start(200)

            ## 调用气泡信息
            self.guid_title = BubbleMessage('   欢迎使用方德AI助手', '', MessageType.Guid, 14, is_send=False)
            self.guid_content = BubbleMessage('作为您的智能助手，我将为您提供个性化的操作系统体验,帮助您完成各种任务、回答问题、提供实用信息，以及提供智能建议和提示。', '',MessageType.Guid, 11, is_send=False)
            

            self.guid_title.setFixedHeight(40)
            self.guid_content.setFixedHeight(85)

            self.verticalLayout = QtWidgets.QVBoxLayout(self)
            self.verticalLayout.setContentsMargins(0, 20, 0, 0)
            self.verticalLayout.setSpacing(10)

            self.verticalLayout.addWidget(self.imageLabel)
            self.verticalLayout.addWidget(self.guid_title)
            self.verticalLayout.addWidget(self.guid_content)

            self.chatGuidWgt = OptionWidget()
            self.chatGuidWgt.set_ui('./data/icon_fdzs.png', '智能问答', '有啥不明白的都可以问我，需要设置系统也请交给我。')
            self.verticalLayout.addWidget(self.chatGuidWgt)
            self.paintGuidWgt = OptionWidget()
            self.paintGuidWgt.set_ui('./data/icon_htzz.png', '绘画制作', '擅长处理各种图形任务，您需要什么尽管吩咐。\n')
            self.verticalLayout.addWidget(self.paintGuidWgt)
            self.meetingMinutsGuidWgt = OptionWidget()
            self.meetingMinutsGuidWgt.set_ui('./data/icon_hyms.png', '会议记录', '帮助您智能管理会议，不漏掉工作中每一个细节。\n')
            self.verticalLayout.addWidget(self.meetingMinutsGuidWgt)
            
            self.spacerItem = QSpacerItem(45, 45, QSizePolicy.Minimum, QSizePolicy.Expanding)
            self.verticalLayout.addItem(self.spacerItem)

            self.chatGuidWgt.clicked.connect(self.chat_model_signal)
            self.paintGuidWgt.clicked.connect(self.paint_model_signal)
            self.meetingMinutsGuidWgt.clicked.connect(self.meeting_minuts_model_signal)
            
            
        def switchViewType(self):
            if PublicTypes.viewType == "sidebar":
                self.verticalLayout.setSpacing(10)
                # self.chatGuidWgt.setFixedWidth(440)
                # self.paintGuidWgt.setFixedWidth(440)
                # self.meetingMinutsGuidWgt.setFixedWidth(440)
                self.guid_title.setFixedWidth(440)
                self.guid_content.setFixedWidth(440)
                self.guid_title.setFixedHeight(40)
                self.guid_content.setFixedHeight(85)
                self.guid_content.message.setStyleSheet(
                    '''
                    background-color: transparent;
                    color:#FFFFFF
                    '''
                )
                self.guid_title.message.setStyleSheet(
                    '''
                    background-color: transparent;
                    color:#FFFFFF
                    '''
                )
                self.chatGuidWgt.ui.titleLabel.setStyleSheet("color: #FFFFFF;\n""font: 16px bold \"方正黑体_GBK\";")
                self.paintGuidWgt.ui.titleLabel.setStyleSheet("color: #FFFFFF;\n""font: 16px bold \"方正黑体_GBK\";")
                self.meetingMinutsGuidWgt.ui.titleLabel.setStyleSheet("color: #FFFFFF;\n""font: 16px bold \"方正黑体_GBK\";")
                self.verticalLayout.setAlignment(self.chatGuidWgt,Qt.AlignLeft)
                self.verticalLayout.setAlignment(self.paintGuidWgt,Qt.AlignLeft)
                self.verticalLayout.setAlignment(self.meetingMinutsGuidWgt,Qt.AlignLeft)
                self.verticalLayout.setAlignment(self.guid_title,Qt.AlignLeft)
                self.verticalLayout.setAlignment(self.guid_content,Qt.AlignLeft)
                # self.setFixedHeight(600)
                h = self.height - 380
                self.setFixedHeight(h)
                self.setFixedWidth(480 - 36)
                

                
            elif PublicTypes.viewType == "windows":
                self.verticalLayout.setSpacing(10)
                self.chatGuidWgt.setFixedWidth(440)
                self.paintGuidWgt.setFixedWidth(440)
                self.meetingMinutsGuidWgt.setFixedWidth(440)
                self.guid_title.setFixedWidth(480)
                self.guid_content.setFixedWidth(480)
                self.guid_title.setFixedHeight(40)
                self.guid_title.message.setStyleSheet(
                    '''
                    background-color: transparent;
                    color:#000000
                    '''
                )
                self.guid_content.setFixedHeight(80)
                self.guid_content.message.setStyleSheet(
                    '''
                    background-color: transparent;
                    color:#000000
                    '''
                )
                self.chatGuidWgt.ui.titleLabel.setStyleSheet("color: #000000;\n""font: 16px bold \"方正黑体_GBK\";")
                self.paintGuidWgt.ui.titleLabel.setStyleSheet("color: #000000;\n""font: 16px bold \"方正黑体_GBK\";")
                self.meetingMinutsGuidWgt.ui.titleLabel.setStyleSheet("color: #000000;\n""font: 16px bold \"方正黑体_GBK\";")
                self.verticalLayout.setAlignment(self.chatGuidWgt,Qt.AlignCenter)
                self.verticalLayout.setAlignment(self.paintGuidWgt,Qt.AlignCenter)
                self.verticalLayout.setAlignment(self.meetingMinutsGuidWgt,Qt.AlignCenter)
                self.verticalLayout.setAlignment(self.guid_title,Qt.AlignCenter)
                self.verticalLayout.setAlignment(self.guid_content,Qt.AlignCenter)
                self.setFixedHeight(560)
                self.setFixedWidth(1164)
            
                
            else:
                raise NotImplementedError("illegal type")   
            
        def updateImage(self):
            if self.timecounter < 15:
                self.timecounter= self.timecounter+1
            else:
                self.timecounter=0
            if self.current_image_index1 < len(self.image_files1) and self.timecounter<4:
                pixmap = QPixmap(self.image_files1[self.current_image_index1])
                
                self.imageLabel.setMinimumSize(pixmap.size())

                self.imageLabel.setPixmap(pixmap)
                self.current_image_index1 = (self.current_image_index1 + 1) % len(self.image_files1)



class OptionWidget(QWidget):
    clicked = pyqtSignal()
    def __init__(self):
        super().__init__()
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.ui = Ui_optWgt()
        self.ui.setupUi(self)
    def set_ui(self, icon_path, title, content):
        self.ui.iconLabel.setPixmap(QtGui.QPixmap(icon_path))
        self.ui.titleLabel.setText(title)
        self.ui.contentLabel.setText(content)
        
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.clicked.emit()

    def paintEvent(self, event):
        painter = QPainter(self)
        # print(self.size())
        painter.setOpacity(0.80)
        painter.setRenderHint(QPainter.Antialiasing)  # 设置抗锯齿，让圆角更加平滑
        # # 定义画笔和填充
        if PublicTypes.viewType == "sidebar":
            painter.setBrush(QBrush(QColor(12,21,34)))  # 画刷颜色设置为白色
            painter.setPen(QPen(QColor(12, 21, 34), 0))  # 画笔颜色和粗细
        elif PublicTypes.viewType == "windows":
            painter.setBrush(QBrush(QColor(255,255,255)))  # 画刷颜色设置为白色
            painter.setPen(QPen(QColor(255, 255, 255), 0))  # 画笔颜色和粗细
        else:
            raise NotImplementedError("illegal type")   
        # # 绘制圆角矩形
        painter.drawRoundedRect(self.rect(), 10, 10)  # 设置边角为10像素的圆度


class Ui_optWgt(object):
    def setupUi(self, optWgt):
        # optWgt.resize(492, 108)
        # optWgt.setMinimumSize(QtCore.QSize(360, 108))
        self.horizontalLayout = QtWidgets.QHBoxLayout(optWgt)
        self.horizontalLayout.setContentsMargins(24, 10, 24, 18)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.iconLabel = QtWidgets.QLabel(optWgt)
        self.iconLabel.setMinimumSize(QtCore.QSize(72, 72))
        self.iconLabel.setMaximumSize(QtCore.QSize(72, 72))
        self.iconLabel.setText("")
        self.iconLabel.setObjectName("iconLabel")
        self.horizontalLayout.addWidget(self.iconLabel)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(9)
        self.verticalLayout.setObjectName("verticalLayout")
        self.titleLabel = QtWidgets.QLabel(optWgt)
        self.titleLabel.setMinimumSize(QtCore.QSize(64, 16))
        font = QtGui.QFont()
        font.setFamily("13 方正黑体_GBK")
        # font.setPointSize(-1)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.titleLabel.setFont(font)
        # self.titleLabel.setAutoFillBackground(False)
        self.titleLabel.setStyleSheet("color: #FFFFFF;\n"
"font: 16px bold \"方正黑体_GBK\";")
        self.titleLabel.setText("")
        self.titleLabel.setObjectName("titleLabel")
        self.verticalLayout.addWidget(self.titleLabel)
        self.contentLabel = QtWidgets.QLabel(optWgt)
        self.contentLabel.setMinimumSize(QtCore.QSize(216, 32))
        self.contentLabel.setStyleSheet("color: #B3B3B3;\n"
"font: 14px \"方正黑体_GBK\";")
        self.contentLabel.setText("")
        self.contentLabel.setWordWrap(True)
        self.contentLabel.setObjectName("contentLabel")
        self.verticalLayout.addWidget(self.contentLabel)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(optWgt)
        QtCore.QMetaObject.connectSlotsByName(optWgt)

    def retranslateUi(self, optWgt):
        _translate = QtCore.QCoreApplication.translate
        optWgt.setWindowTitle(_translate("optWgt", "Form"))

#  测试布局和交互效果
# from PyQt5.QtWidgets import *
# from PyQt5.QtCore import *
# from PyQt5.QtGui import *
#
# if __name__ == '__main__':
#     app = QApplication([])
#     widget = PreWidget()
#     widget.show()
#     app.exec_()


